@@ -29,14 +29,16 @@
 #define __SWI_PROLOG__	/* normally defined by the swipl-ld compiler driver */
 #endif
 
-#if defined(_MSC_VER) && !defined(__WINDOWS__)
+#ifndef __WINDOWS__
+#if defined(_MSC_VER) || defined(__MINGW32__)
 #define __WINDOWS__ 1
 #endif
+#endif
 
 #include <stdarg.h>
 #include <stdlib.h>			/* get size_t */
 #include <stddef.h>
-#ifdef __WINDOWS__
+#ifdef _MSC_VER
 typedef __int64 int64_t;
 typedef unsigned __int64 uint64_t;
 #if (_MSC_VER < 1300)
@@ -54,7 +56,7 @@ extern "C" {
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 51004
+#define PLVERSION 51005
 #endif
 
 		 /*******************************
@@ -88,21 +90,21 @@ duplicated this stuff.
 # ifdef PL_KERNEL
 #define PL_EXPORT(type)		__declspec(dllexport) type
 #define PL_EXPORT_DATA(type)	__declspec(dllexport) type
-#define install_t	 	void
+#define install_t		void
 # else
 #  ifdef __BORLANDC__
-#define PL_EXPORT(type)	 	type _stdcall
+#define PL_EXPORT(type)		type _stdcall
 #define PL_EXPORT_DATA(type)	extern type
 #  else
-#define PL_EXPORT(type)	 	extern type
+#define PL_EXPORT(type)		extern type
 #define PL_EXPORT_DATA(type)	__declspec(dllimport) type
 #  endif
-#define install_t	 	__declspec(dllexport) void
+#define install_t		__declspec(dllexport) void
 # endif
 #else /*HAVE_DECLSPEC*/
-#define PL_EXPORT(type)	 	extern type
+#define PL_EXPORT(type)		extern type
 #define PL_EXPORT_DATA(type)	extern type
-#define install_t	 	void
+#define install_t		void
 #endif /*HAVE_DECLSPEC*/
 #endif /*_PL_EXPORT_DONE*/
 
@@ -143,7 +145,7 @@ typedef uintptr_t	PL_atomic_t;	/* same a word */
 typedef uintptr_t	foreign_t;	/* return type of foreign functions */
 typedef wchar_t	        pl_wchar_t;	/* Prolog wide character */
 #ifdef __cplusplus
-typedef void * 		pl_function_t;	/* can only pass function as void * */
+typedef void *		pl_function_t;	/* can only pass function as void * */
 #else
 typedef foreign_t	(*pl_function_t)(); /* foreign language functions */
 #endif
@@ -256,7 +258,7 @@ typedef union
 
 PL_EXPORT(foreign_t)	_PL_retry(intptr_t);
 PL_EXPORT(foreign_t)	_PL_retry_address(void *);
-PL_EXPORT(int)	 	PL_foreign_control(control_t);
+PL_EXPORT(int)		PL_foreign_control(control_t);
 PL_EXPORT(intptr_t)	PL_foreign_context(control_t);
 PL_EXPORT(void *)	PL_foreign_context_address(control_t);
 
@@ -521,21 +523,21 @@ PL_EXPORT(int)		PL_get_attr(term_t v, term_t a);
 		 *	      ERRORS		*
 		 *******************************/
 
-PL_EXPORT(int) 		PL_get_atom_ex(term_t t, atom_t *a);
-PL_EXPORT(int) 		PL_get_integer_ex(term_t t, int *i);
-PL_EXPORT(int) 		PL_get_long_ex(term_t t, long *i);
-PL_EXPORT(int) 		PL_get_int64_ex(term_t t, int64_t *i);
-PL_EXPORT(int) 		PL_get_intptr_ex(term_t t, intptr_t *i);
-PL_EXPORT(int) 		PL_get_size_ex(term_t t, size_t *i);
-PL_EXPORT(int) 		PL_get_bool_ex(term_t t, int *i);
-PL_EXPORT(int) 		PL_get_float_ex(term_t t, double *f);
-PL_EXPORT(int) 		PL_get_char_ex(term_t t, int *p, int eof);
-PL_EXPORT(int) 		PL_unify_bool_ex(term_t t, int val);
-PL_EXPORT(int) 		PL_get_pointer_ex(term_t t, void **addrp);
-PL_EXPORT(int) 		PL_unify_list_ex(term_t l, term_t h, term_t t);
-PL_EXPORT(int) 		PL_unify_nil_ex(term_t l);
-PL_EXPORT(int) 		PL_get_list_ex(term_t l, term_t h, term_t t);
-PL_EXPORT(int) 		PL_get_nil_ex(term_t l);
+PL_EXPORT(int)		PL_get_atom_ex(term_t t, atom_t *a);
+PL_EXPORT(int)		PL_get_integer_ex(term_t t, int *i);
+PL_EXPORT(int)		PL_get_long_ex(term_t t, long *i);
+PL_EXPORT(int)		PL_get_int64_ex(term_t t, int64_t *i);
+PL_EXPORT(int)		PL_get_intptr_ex(term_t t, intptr_t *i);
+PL_EXPORT(int)		PL_get_size_ex(term_t t, size_t *i);
+PL_EXPORT(int)		PL_get_bool_ex(term_t t, int *i);
+PL_EXPORT(int)		PL_get_float_ex(term_t t, double *f);
+PL_EXPORT(int)		PL_get_char_ex(term_t t, int *p, int eof);
+PL_EXPORT(int)		PL_unify_bool_ex(term_t t, int val);
+PL_EXPORT(int)		PL_get_pointer_ex(term_t t, void **addrp);
+PL_EXPORT(int)		PL_unify_list_ex(term_t l, term_t h, term_t t);
+PL_EXPORT(int)		PL_unify_nil_ex(term_t l);
+PL_EXPORT(int)		PL_get_list_ex(term_t l, term_t h, term_t t);
+PL_EXPORT(int)		PL_get_nil_ex(term_t l);
 
 PL_EXPORT(int)		PL_instantiation_error(term_t culprit);
 PL_EXPORT(int)		PL_representation_error(const char *resource);
@@ -544,6 +546,8 @@ PL_EXPORT(int)		PL_domain_error(const char *expected, term_t culprit);
 PL_EXPORT(int)		PL_existence_error(const char *type, term_t culprit);
 PL_EXPORT(int)		PL_permission_error(const char *operation,
 					    const char *type, term_t culprit);
+PL_EXPORT(int)		PL_resource_error(const char *resource);
+
 
 		 /*******************************
 		 *	       BLOBS		*
@@ -742,7 +746,7 @@ UNICODE file functions.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define REP_ISO_LATIN_1 0x0000		/* output representation */
-#define REP_UTF8 	0x1000
+#define REP_UTF8	0x1000
 #define REP_MB		0x2000
 #ifdef __WINDOWS__
 #define REP_FN		REP_UTF8
@@ -760,9 +764,9 @@ UNICODE file functions.
 
 					/* Make IOSTREAM known to Prolog */
 #define PL_open_stream  PL_unify_stream	/* compatibility */
-PL_EXPORT(int)  	PL_unify_stream(term_t t, IOSTREAM *s);
-PL_EXPORT(int)  	PL_get_stream_handle(term_t t, IOSTREAM **s);
-PL_EXPORT(int) 		PL_release_stream(IOSTREAM *s);
+PL_EXPORT(int)		PL_unify_stream(term_t t, IOSTREAM *s);
+PL_EXPORT(int)		PL_get_stream_handle(term_t t, IOSTREAM **s);
+PL_EXPORT(int)		PL_release_stream(IOSTREAM *s);
 PL_EXPORT(IOSTREAM *)	PL_open_resource(module_t m,
 					 const char *name,
 					 const char *rc_class,
@@ -792,6 +796,8 @@ PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 	 PL_WRT_ATTVAR_WRITE | \
 	 PL_WRT_ATTVAR_PORTRAY)
 #define PL_WRT_BLOB_PORTRAY	0x400	/* Use portray to emit non-text blobs */
+#define PL_WRT_NO_CYCLES	0x800	/* Never emit @(Template,Subst) */
+#define PL_WRT_LIST	       0x1000	/* Write [...], even with ignoreops */
 
 PL_EXPORT(int) PL_write_term(IOSTREAM *s,
 			     term_t term,
@@ -876,11 +882,11 @@ typedef void (*PL_abort_hook_t)(void);
 typedef void (*PL_initialise_hook_t)(int argc, char **argv);
 typedef int  (*PL_agc_hook_t)(atom_t a);
 
-PL_EXPORT(PL_dispatch_hook_t) 	PL_dispatch_hook(PL_dispatch_hook_t);
-PL_EXPORT(void)	       		PL_abort_hook(PL_abort_hook_t);
-PL_EXPORT(void)	       		PL_initialise_hook(PL_initialise_hook_t);
-PL_EXPORT(int)		      	PL_abort_unhook(PL_abort_hook_t);
-PL_EXPORT(PL_agc_hook_t)      	PL_agc_hook(PL_agc_hook_t);
+PL_EXPORT(PL_dispatch_hook_t)	PL_dispatch_hook(PL_dispatch_hook_t);
+PL_EXPORT(void)			PL_abort_hook(PL_abort_hook_t);
+PL_EXPORT(void)			PL_initialise_hook(PL_initialise_hook_t);
+PL_EXPORT(int)			PL_abort_unhook(PL_abort_hook_t);
+PL_EXPORT(PL_agc_hook_t)	PL_agc_hook(PL_agc_hook_t);
 
 
 		/********************************
@@ -894,7 +900,7 @@ PL_EXPORT(void) (*PL_signal(int sig, void (*func)(int)))(int);
 PL_EXPORT(void) PL_interrupt(int sig);
 PL_EXPORT(int)  PL_raise(int sig);
 PL_EXPORT(int)  PL_handle_signals(void);
-PL_EXPORT(int) 	PL_get_signum_ex(term_t sig, int *n);
+PL_EXPORT(int)	PL_get_signum_ex(term_t sig, int *n);
 
 
 		/********************************
@@ -991,7 +997,7 @@ PL_EXPORT(int)		PL_destroy_engine(PL_engine_t engine);
 		 *******************************/
 
 typedef struct
-{ int	(*unify)(term_t t, void *handle); 	/* implementation --> Prolog */
+{ int	(*unify)(term_t t, void *handle);	/* implementation --> Prolog */
   int   (*get)(term_t t, void **handle);	/* Prolog --> implementation */
   void	(*activate)(int active);		/* (de)activate */
   intptr_t	magic;					/* PROFTYPE_MAGIC */
@@ -1041,8 +1047,24 @@ PL_EXPORT(int) _PL_put_xpce_reference_a(term_t t, atom_t name);
 		 *         TRACE SUPPORT	*
 		 *******************************/
 
-PL_EXPORT(int) PL_walk_prolog_stack(void *ref, char* buf, size_t len, void** nextref);
+#ifndef _PL_INCLUDE_H
+typedef void *QueryFrame;
+typedef void *LocalFrame;
+typedef void *Code;
+#endif
 
+typedef struct pl_context_t
+{ PL_engine_t   ld;			/* Engine */
+  QueryFrame	qf;			/* Current query */
+  LocalFrame	fr;			/* Current localframe */
+  Code		pc;			/* Code pointer */
+  void *	reserved[10];		/* Reserved for extensions */
+} pl_context_t;
+
+PL_EXPORT(int)	PL_get_context(struct pl_context_t *c, int thead_id);
+PL_EXPORT(int)	PL_step_context(struct pl_context_t *c);
+PL_EXPORT(int)	PL_describe_context(struct pl_context_t *c,
+				    char *buf, size_t len);
 
 #ifdef __cplusplus
 }
