@@ -61,6 +61,8 @@ index((maplist), 2, apply, apply).
 index((maplist), 3, apply, apply).
 index((maplist), 4, apply, apply).
 index((maplist), 5, apply, apply).
+index((arithmetic_function), 1, arithmetic, arithmetic).
+index((arithmetic_expression_value), 2, arithmetic, arithmetic).
 index((empty_assoc), 1, assoc, assoc).
 index((is_assoc), 1, assoc, assoc).
 index((assoc_to_list), 2, assoc, assoc).
@@ -307,6 +309,7 @@ index((listing), 0, prolog_listing, listing).
 index((listing), 1, prolog_listing, listing).
 index((portray_clause), 1, prolog_listing, listing).
 index((portray_clause), 2, prolog_listing, listing).
+index((portray_clause), 3, prolog_listing, listing).
 index((member), 2, lists, lists).
 index((append), 2, lists, lists).
 index((append), 3, lists, lists).
@@ -320,6 +323,7 @@ index((delete), 3, lists, lists).
 index((nth0), 3, lists, lists).
 index((nth1), 3, lists, lists).
 index((last), 2, lists, lists).
+index((same_length), 2, lists, lists).
 index((reverse), 2, lists, lists).
 index((permutation), 2, lists, lists).
 index((flatten), 2, lists, lists).
@@ -377,6 +381,10 @@ index((select_option), 3, swi_option, option).
 index((select_option), 4, swi_option, option).
 index((merge_options), 3, swi_option, option).
 index((meta_options), 3, swi_option, option).
+index((opt_parse), 5, optparse, optparse).
+index((opt_parse), 4, optparse, optparse).
+index((opt_arguments), 3, optparse, optparse).
+index((opt_help), 2, optparse, optparse).
 index((is_ordset), 1, ordsets, ordsets).
 index((list_to_ord_set), 2, ordsets, ordsets).
 index((ord_add_element), 3, ordsets, ordsets).
@@ -413,6 +421,7 @@ index((group_pairs_by_key), 2, pairs, pairs).
 index((transpose_pairs), 2, pairs, pairs).
 index((map_list_to_pairs), 3, pairs, pairs).
 index((persistent), 1, persistency, persistency).
+index((current_persistent_predicate), 1, persistency, persistency).
 index((db_attach), 2, persistency, persistency).
 index((db_sync), 1, persistency, persistency).
 index((db_sync_all), 1, persistency, persistency).
@@ -460,6 +469,7 @@ index((print_prolog_backtrace), 2, prolog_stack, prolog_stack).
 index((backtrace), 1, prolog_stack, prolog_stack).
 index((xref_source), 1, prolog_xref, prolog_xref).
 index((xref_called), 3, prolog_xref, prolog_xref).
+index((xref_called), 4, prolog_xref, prolog_xref).
 index((xref_defined), 3, prolog_xref, prolog_xref).
 index((xref_definition_line), 2, prolog_xref, prolog_xref).
 index((xref_exported), 2, prolog_xref, prolog_xref).
@@ -600,6 +610,7 @@ index((read_file_to_codes), 3, read_util, readutil).
 index((read_file_to_terms), 3, read_util, readutil).
 index((record), 1, record, record).
 index((current_record), 2, record, record).
+index((current_record_predicate), 2, record, record).
 index((rewrite), 2, rewrite, rewrite).
 index((rew_term_expansion), 2, rewrite, rewrite).
 index((rew_goal_expansion), 2, rewrite, rewrite).
@@ -616,6 +627,7 @@ index((save_settings), 1, settings, settings).
 index((current_setting), 1, settings, settings).
 index((setting_property), 2, settings, settings).
 index((list_settings), 0, settings, settings).
+index((list_settings), 1, settings, settings).
 index((convert_setting_text), 3, settings, settings).
 index((load_sgml_file), 2, sgml, sgml).
 index((load_xml_file), 2, sgml, sgml).
@@ -778,6 +790,7 @@ index((tnodebug), 1, thread_util, threadutil).
 index((tprofile), 1, thread_util, threadutil).
 index((alarm), 3, time, time).
 index((alarm), 4, time, time).
+index((alarm_at), 3, time, time).
 index((alarm_at), 4, time, time).
 index((remove_alarm), 1, time, time).
 index((install_alarm), 1, time, time).
@@ -805,6 +818,20 @@ index((transpose), 2, ugraphs, ugraphs).
 index((vertices), 2, ugraphs, ugraphs).
 index((vertices_edges_to_ugraph), 3, ugraphs, ugraphs).
 index((ugraph_union), 3, ugraphs, ugraphs).
+index((getuid), 1, uid, uid).
+index((getgid), 1, uid, uid).
+index((geteuid), 1, uid, uid).
+index((getegid), 1, uid, uid).
+index((user_info), 2, uid, uid).
+index((group_info), 2, uid, uid).
+index((user_data), 3, uid, uid).
+index((group_data), 3, uid, uid).
+index((setuid), 1, uid, uid).
+index((setgid), 1, uid, uid).
+index((seteuid), 1, uid, uid).
+index((setegid), 1, uid, uid).
+index((set_user_and_group), 1, uid, uid).
+index((set_user_and_group), 2, uid, uid).
 index((fork), 1, unix, unix).
 index((exec), 1, unix, unix).
 index((wait), 2, unix, unix).
