@@ -1336,8 +1336,8 @@ print_char('&', Out) :- !, write(Out, '\\&').
 print_char('#', Out) :- !, write(Out, '\\#').
 print_char('%', Out) :- !, write(Out, '\\%').
 print_char('\\',Out) :- !, write(Out, '\\bsl{}').
-print_char('^', Out) :- !, write(Out, '$\\wedge$').
-print_char('|', Out) :- !, write(Out, '{\\tt\\string|}').
+print_char('^', Out) :- !, write(Out, '\\Shat{}').
+print_char('|', Out) :- !, write(Out, '\\Sbar{}').
 print_char(C,   Out) :- put_char(Out, C).
 
 
