@@ -70,7 +70,7 @@
 :- use_module(library(option)).
 :- use_module(library(pairs)).
 :- use_module(library(sgml)).		% Quote output
-:- use_module(library(url)).
+:- use_module(library(uri)).
 :- use_module(library(quintus)).	% for meta_predicate/1
 :- set_prolog_flag(generate_debug_info, false).
 
@@ -582,6 +582,8 @@ name(Name) -->
 %	  Emit URL-encoded version of V.  See www_form_encode/2.
 %	  * An option list
 %	  Emit ?Name1=encode(Value1)&Name2=encode(Value2) ...
+%	  * A term Format-Arguments
+%	  Use format/3 and emit the result as quoted value.
 %
 %	The hook html_write:expand_attribute_value//1 can  be defined to
 %	provide additional `function like'   translations.  For example,
@@ -592,24 +594,36 @@ name(Name) -->
 :- multifile
 	expand_attribute_value//1.
 
-attribute_value(Var) -->
+attribute_value(List) -->
+	{ is_list(List) },
+	attribute_value_m(List).
+attribute_value(Value) -->
+	attribute_value_s(Value).
+
+% emit a single attribute value
+
+attribute_value_s(Var) -->
 	{ var(Var), !,
 	  instantiation_error(Var)
 	}.
-attribute_value(A+B) --> !,
+attribute_value_s(A+B) --> !,
 	attribute_value(A),
-	attribute_value(B).
-attribute_value([]) --> !.
-attribute_value(List) -->
-	{ is_list(List) }, !,
-	[ ? ],
-	search_parameters(List).
-attribute_value(encode(Value)) --> !,
-	{ www_form_encode(Value, Encoded) },
+	(   { is_list(B) }
+	->  (   { B == [] }
+	    ->	[]
+	    ;	[?], search_parameters(B)
+	    )
+	;   attribute_value(B)
+	).
+attribute_value_s(encode(Value)) --> !,
+	{ uri_encoded(query_value, Value, Encoded) },
 	[ Encoded ].
-attribute_value(Value) -->
+attribute_value_s(Value) -->
 	expand_attribute_value(Value), !.
-attribute_value(Value) -->
+attribute_value_s(Fmt-Args) --> !,
+	{ format(string(Value), Fmt, Args) },
+	html_quoted_attribute(Value).
+attribute_value_s(Value) -->
 	html_quoted_attribute(Value).
 
 search_parameters([H|T]) -->
@@ -636,6 +650,26 @@ search_parameter(Term) -->
 	{ domain_error(search_parameter, Term)
 	}.
 
+%%	attribute_value_m(+List)//
+%
+%	Used for multi-valued attributes, such as class-lists.  E.g.,
+%
+%	  ==
+%		body(class([c1, c2]), Body)
+%	  ==
+%
+%	  Emits =|<body class="c1 c2"> ...|=
+
+attribute_value_m([]) -->
+	[].
+attribute_value_m([H|T]) -->
+	attribute_value_s(H),
+	(   { T == [] }
+	->  []
+	;   [' '],
+	    attribute_value_m(T)
+	).
+
 
 		 /*******************************
 		 *	   QUOTING RULES	*
@@ -882,14 +916,14 @@ post_close(_) -->
 %			close-tag or =empty= to indicate the element has
 %			no content model.
 %
-% 	@tbd	Complete table
+%	@tbd	Complete table
 
 :- multifile
 	layout/3.
 
 layout(table,	   2-1,	1-2).
 layout(blockquote, 2-1,	1-2).
-layout(pre, 	   2-1,	1-2).
+layout(pre,	   2-1,	1-2).
 layout(center,	   2-1,	1-2).
 layout(dl,	   2-1,	1-2).
 layout(ul,	   1-1,	1-1).
@@ -1065,7 +1099,13 @@ reply_html_page(Style, Head, Body) :-
 %	rule takes HTML content as  argument.   It  has  two effects. It
 %	emits  the  appropriate  meta_predicate/1    and  instructs  the
 %	built-in editor (PceEmacs) to provide   proper colouring for the
-%	arguments.
+%	arguments.  The  arguments  in  Head  are    the   same  as  for
+%	meta_predicate or can be constant =html=.  For example:
+%
+%	  ==
+%	  :- html_meta
+%		page(html,html,?,?).
+%	  ==
 
 html_meta(Spec) :-
 	throw(error(context_error(nodirective, html_meta(Spec)), _)).
@@ -1076,7 +1116,7 @@ html_meta_decls(Var, _, _) :-
 html_meta_decls((A,B), (MA,MB), [MH|T]) :- !,
 	html_meta_decl(A, MA, MH),
 	html_meta_decls(B, MB, T).
-html_meta_decls(A, MA, MH) :-
+html_meta_decls(A, MA, [MH]) :-
 	html_meta_decl(A, MA, MH).
 
 html_meta_decl(Head, MetaHead,
@@ -1107,6 +1147,9 @@ html_meta_colours(Head, Goal, built_in-Colours) :-
 
 meta_colours(html, HTML, Colours) :- !,
 	html_colours(HTML, Colours).
+meta_colours(I, _, Colours) :-
+	integer(I), I>=0, !,
+	Colours = meta(I).
 meta_colours(_, _, classify).
 
 html_meta_called(Head, Goal, Called) :-
@@ -1118,6 +1161,11 @@ meta_called([], [], Called, Called).
 meta_called([html|MT], [A|AT], Called, Tail) :- !,
 	phrase(called_by(A), Called, Tail1),
 	meta_called(MT, AT, Tail1, Tail).
+meta_called([0|MT], [A|AT], [A|CT0], CT) :- !,
+	meta_called(MT, AT, CT0, CT).
+meta_called([I|MT], [A|AT], [A+I|CT0], CT) :-
+	integer(I), I>0, !,
+	meta_called(MT, AT, CT0, CT).
 meta_called([_|MT], [_|AT], Called, Tail) :- !,
 	meta_called(MT, AT, Called, Tail).
 
@@ -1307,6 +1355,14 @@ called_by(Term, M) -->
 called_by(_, _) -->
 	[].
 
+:- multifile
+	prolog:hook/1.
+
+prolog:hook(body(_,_,_)).
+prolog:hook(body(_,_,_,_)).
+prolog:hook(head(_,_,_)).
+prolog:hook(head(_,_,_,_)).
+
 
 		 /*******************************
 		 *	      MESSAGES		*
