@@ -43,6 +43,7 @@
 
 	    http_read_header/2,		% +Fd, -Header
 	    http_parse_header/2,	% +Codes, -Header
+	    http_parse_header_value/3,	% +Header, +HeaderValue, -MediaTypes
 	    http_join_headers/3,	% +Default, +InHdr, -OutHdr
 	    http_update_encoding/3,	% +HeaderIn, -Encoding, -HeaderOut
 	    http_update_connection/4,	% +HeaderIn, +Request, -Connection, -HeaderOut
@@ -52,6 +53,7 @@
 :- use_module(library(debug)).
 :- use_module(library(lists)).
 :- use_module(library(url)).
+:- use_module(library(uri)).
 :- use_module(library(memfile)).
 :- use_module(library(settings)).
 :- use_module(library(error)).
@@ -143,7 +145,7 @@ http_read_reply_header(In, [input(In)|Reply]) :-
 %		typical output from a CGI script.
 %
 %		* Status
-%		HTTP status report and defined by http_status_reply/3.
+%		HTTP status report and defined by http_status_reply/4.
 %
 %	@param HdrExtra provides additional reply-header fields, encoded
 %	       as Name(Value). It can also contain a field
@@ -177,7 +179,7 @@ http_reply(Status, Out, HdrExtra, Code) :-
 %%	http_reply_data(+Data, +Out, +HdrExtra, -Code) is semidet.
 %
 %	Fails if Data is not a defined   reply-data format, but a status
-%	term. See http_reply/3 and http_status_reply/3.
+%	term. See http_reply/3 and http_status_reply/4.
 %
 %	@error Various I/O errors.
 
@@ -562,7 +564,7 @@ transfer(Header, Transfer) :-
 
 %%	content_length_in_encoding(+Encoding, +In, -Bytes)
 %
-%	Determine hom much bytes are required to represent the data from
+%	Determine hom many bytes are required to represent the data from
 %	stream In using the given encoding.  Fails if the data cannot be
 %	represented with the given encoding.
 
@@ -596,6 +598,13 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %	  * html(+Tokens)
 %	  Result of html//1 from html_write.pl
 %
+%	  * xml(+Term)
+%	  Post the result of xml_write/3 using the Mime-type
+%	  =|text/xml|=
+%
+%	  * xml(+Type, +Term)
+%	  Post the result of xml_write/3 using the given Mime-type.
+%
 %	  * file(+File)
 %	  Send contents of a file. Mime-type is determined by
 %	  file_mime_type/2.
@@ -616,7 +625,7 @@ content_length_in_encoding(Enc, Stream, Bytes) :-
 %
 %	  * form(+ListOfParameter)
 %	  Send data of the MIME type application/x-www-form-urlencoded as
-% 	  produced by browsers issuing a POST request from an HTML form.
+%	  produced by browsers issuing a POST request from an HTML form.
 %	  ListOfParameter is a list of Name=Value or Name(Value).
 %
 %	  * form_data(+ListOfData)
@@ -637,6 +646,16 @@ http_post_data(html(HTML), Out, HdrExtra) :-
 	phrase(post_header(html(HTML), HdrExtra), Header),
 	format(Out, '~s', [Header]),
 	print_html(Out, HTML).
+http_post_data(xml(XML), Out, HdrExtra) :-
+	http_post_data(xml(text/xml, XML), Out, HdrExtra).
+http_post_data(xml(Type, XML), Out, HdrExtra) :-
+	setup_call_cleanup(new_memory_file(MemFile),
+			   (   setup_call_cleanup(open_memory_file(MemFile, write, MemOut),
+						  xml_write(MemOut, XML, []),
+						  close(MemOut)),
+			       http_post_data(memory_file(Type, MemFile), Out, HdrExtra)
+			   ),
+			   free_memory_file(MemFile)).
 http_post_data(file(File), Out, HdrExtra) :- !,
 	(   file_mime_type(File, Type)
 	->  true
@@ -649,6 +668,12 @@ http_post_data(file(Type, File), Out, HdrExtra) :- !,
 	open(File, read, In, [type(binary)]),
 	call_cleanup(copy_stream_data(In, Out),
 		     close(In)).
+http_post_data(memory_file(Type, Handle), Out, HdrExtra) :- !,
+	phrase(post_header(memory_file(Type, Handle), HdrExtra), Header),
+	format(Out, '~s', [Header]),
+	setup_call_cleanup(open_memory_file(Handle, read, In, [encoding(octet)]),
+			   copy_stream_data(In, Out),
+			   close(In)).
 http_post_data(codes(Codes), Out, HdrExtra) :- !,
 	http_post_data(codes(text/plain, Codes), Out, HdrExtra).
 http_post_data(codes(Type, Codes), Out, HdrExtra) :- !,
@@ -727,6 +752,11 @@ post_header(file(Type, File), HdrExtra) -->
 	content_length(file(File), Len),
 	content_type(Type),
 	"\r\n".
+post_header(memory_file(Type, File), HdrExtra) -->
+	header_fields(HdrExtra, Len),
+	content_length(memory_file(File), Len),
+	content_type(Type),
+	"\r\n".
 post_header(cgi_data(Size), HdrExtra) -->
 	header_fields(HdrExtra, Len),
 	content_length(Size, Len),
@@ -998,6 +1028,8 @@ length_of(ascii_string(String), Len) :- !,
 	length(String, Len).
 length_of(file(File), Len) :- !,
 	size_file(File, Len).
+length_of(memory_file(Handle), Len) :- !,
+	size_memory_file(Handle, Len, octet).
 length_of(html(Tokens), Len) :- !,
 	html_print_length(Tokens, Len).
 length_of(Len, Len).
@@ -1080,6 +1112,41 @@ read_field_value([H|T]) -->
 	read_field_value(T).
 
 
+%%	http_parse_header_value(+Field, +Value, -Prolog) is semidet.
+%
+%	Translate Value in a meaningful Prolog   term. Field denotes the
+%	HTTP request field for which we   do  the translation. Supported
+%	fields are:
+%
+%	  * content_length
+%	  * cookie
+%	  * set_cookie
+%	  * host
+%	  * range
+%	  * accept
+%
+%	@error	domain_error(http_request_field, Field) if this
+%		library is not prepared to handle this field (yet).
+
+http_parse_header_value(Field, Value, Prolog) :-
+	valid_field(Field),
+	to_codes(Value, Codes),
+	field_to_prolog(Field, Codes, Prolog).
+
+valid_field(content_length).
+valid_field(cookie).
+valid_field(set_cookie).
+valid_field(host).
+valid_field(range).
+valid_field(accept).
+
+to_codes(In, Codes) :-
+	(   is_list(In)
+	->  Codes = In
+	;   atom_codes(In, Codes)
+	).
+
+
 field_to_prolog(content_length, ValueChars, ContentLength) :- !,
 	number_codes(ContentLength, ValueChars).
 field_to_prolog(cookie, ValueChars, Cookies) :- !,
@@ -1097,6 +1164,8 @@ field_to_prolog(host, ValueChars, Host) :- !,
 	).
 field_to_prolog(range, ValueChars, Range) :-
 	phrase(range(Range), ValueChars), !.
+field_to_prolog(accept, ValueChars, Media) :-
+	parse_accept(ValueChars, Media), !.
 field_to_prolog(_, ValueChars, Atom) :-
 	atom_codes(Atom, ValueChars).
 
@@ -1117,6 +1186,186 @@ set_cookie_options([Name=Value|T]) -->
 	set_cookie_options(T).
 
 
+		 /*******************************
+		 *	  ACCEPT HEADERS	*
+		 *******************************/
+
+:- dynamic accept_cache/2.
+:- volatile accept_cache/2.
+
+parse_accept(Codes, Media) :-
+	atom_codes(Atom, Codes),
+	(   accept_cache(Atom, Media0)
+	->  Media = Media0
+	;   phrase(accept(Media0), Codes),
+	    keysort(Media0, Media1),
+	    pairs_values(Media1, MediaR),
+	    reverse(MediaR, Media2),
+	    assertz(accept_cache(Atom, Media2)),
+	    Media = Media2
+	).
+
+%%	accept(-Media)// is semidet.
+%
+%	Parse an HTTP Accept: header
+
+accept([H|T]) -->
+	blanks,
+	media_range(H),
+	blanks,
+	(   ","
+	->  accept(T)
+	;   {T=[]}
+	).
+
+media_range(s(Quality,Spec)-media(Type, TypeParams, Quality, AcceptExts)) -->
+	media_type(Type),
+	blanks,
+	(   ";"
+	->  blanks,
+	    parameters_and_quality(TypeParams, Quality, AcceptExts)
+	;   { TypeParams = [],
+	      Quality = 1.0,
+	      AcceptExts = []
+	    }
+	),
+	{ rank_specialised(Type, TypeParams, Spec) }.
+
+
+%%	rank_specialised(+Type, +TypeParam, -Key) is det.
+%
+%	Although the specification linked  above   is  unclear, it seems
+%	that  more  specialised  types  must   be  preferred  over  less
+%	specialized ones.
+%
+%	@tbd	Is there an official specification of this?
+
+rank_specialised(Type/SubType, TypeParams, v(VT, VS, VP)) :-
+	var_or_given(Type, VT),
+	var_or_given(SubType, VS),
+	length(TypeParams, VP).
+
+var_or_given(V, Val) :-
+	(   var(V)
+	->  Val = 0
+	;   Val = 1
+	).
+
+media_type(Type/SubType) -->
+	type(Type), "/", type(SubType).
+
+type(_) -->
+	"*", !.
+type(Type) -->
+	token(Type).
+
+parameters_and_quality(Params, Quality, AcceptExts) -->
+	token(Name),
+	blanks, "=", blanks,
+	(   { Name == q }
+	->  float(Quality), blanks,
+	    accept_extensions(AcceptExts),
+	    { Params = [] }
+	;   { Params = [Name=Value|T] },
+	    parameter_value(Value),
+	    blanks,
+	    (	";"
+	    ->	blanks,
+		parameters_and_quality(T, Quality, AcceptExts)
+	    ;	{ T = [],
+		  Quality = 1.0,
+		  AcceptExts = []
+		}
+	    )
+	).
+
+accept_extensions([H|T]) -->
+	";", !,
+	blanks, token(Name), blanks,
+	(   "="
+	->  blanks,
+	    (	token(Value)
+	    ->	[]
+	    ;	quoted_string(Value)
+	    ),
+	    { H = (Name=Value) }
+	;   { H = Name }
+	),
+	blanks,
+	accept_extensions(T).
+accept_extensions([]) -->
+	[].
+
+parameter_value(Value) -->
+	(   token(Value)
+	->  []
+	;   quoted_string(Value)
+	).
+
+
+%%	token(-Name)// is semidet.
+%
+%	Process an HTTP header token from the input.
+
+token(Name) -->
+	token_char(C1),
+	token_chars(Cs),
+	{ atom_codes(Name, [C1|Cs]) }.
+
+token_chars([H|T]) -->
+	token_char(H), !,
+	token_chars(T).
+token_chars([]) --> [].
+
+token_char(C) -->
+	[C],
+	{   \+ ctl(C),
+	    \+ separator_code(C)
+	}, !.
+
+ctl(C) :- between(0,31,C), !.
+ctl(127).
+
+separator_code(0'().
+separator_code(0')).
+separator_code(0'<).
+separator_code(0'>).
+separator_code(0'@).
+separator_code(0',).
+separator_code(0';).
+separator_code(0':).
+separator_code(0'\\).
+separator_code(0'").
+separator_code(0'/).
+separator_code(0'[).
+separator_code(0']).
+separator_code(0'?).
+separator_code(0'=).
+separator_code(0'{).
+separator_code(0'}).
+separator_code(32).
+separator_code(0'\t).
+
+
+%%	quoted_string(-Text)// is semidet.
+%
+%	True if input starts with a quoted string representing Text.
+
+quoted_string(Text) -->
+	"\"",
+	quoted_text(Codes),
+	{ atom_codes(Text, Codes) }.
+
+quoted_text([]) -->
+	"\"", !.
+quoted_text([H|T]) -->
+	"\\", !, [H],
+	quoted_text(T).
+quoted_text([H|T]) -->
+	[H], !,
+	quoted_text(T).
+
+
 %%	header_fields(+Fields, ?ContentLength)// is det.
 %
 %	Process a sequence of  [Name(Value),   ...]  attributes  for the
@@ -1163,6 +1412,13 @@ field_name(Name) -->
 	{ atom_codes(Name, Chars) },
 	wr_field_chars(Chars).
 
+rd_field_chars_no_fold([C|T]) -->
+	[C],
+	{ rd_field_char(C, _) }, !,
+	rd_field_chars_no_fold(T).
+rd_field_chars_no_fold([]) -->
+	[].
+
 rd_field_chars([C0|T]) -->
 	[C],
 	{ rd_field_char(C, C0) }, !,
@@ -1190,21 +1446,21 @@ term_expansion(rd_field_char(_,_), Clauses) :-
 
 rd_field_char(_, _).
 
-wr_field_chars([C|T]) -->
-	[C2], !,
-	{ to_lower(C2, C) },
+wr_field_chars([C|T]) --> !,
+	{ code_type(C, to_lower(U)) },
+	[U],
 	wr_field_chars2(T).
 wr_field_chars([]) -->
 	[].
 
-wr_field_chars2([0'_|T]) --> !,		% 0'
-	"-",
-	wr_field_chars(T).
-wr_field_chars2([C|T]) --> !,
-	[C],
-	wr_field_chars2(T).
-wr_field_chars2([]) -->
-	[].
+wr_field_chars2([]) --> [].
+wr_field_chars2([C|T]) -->		% 0'
+	(   { C == 0'_ }
+	->  "-",
+	    wr_field_chars(T)
+	;   [C],
+	    wr_field_chars2(T)
+	).
 
 %	now
 %%	rfc_date(+Time)
@@ -1245,10 +1501,9 @@ request(Fd, [method(Method),request_uri(ReqURI)|Header]) -->
 	blanks,
 	nonblanks(Query),
 	{ atom_codes(ReqURI, Query),
-	  http_location(Parts, Query),
-	  append(Parts, Header0, Header)
+	  request_uri_parts(ReqURI, Header, Rest)
 	},
-	request_header(Fd, Header0), !.
+	request_header(Fd, Rest), !.
 request(Fd, [unknown(What)|Header]) -->
 	string(What),
 	eos, !,
@@ -1265,6 +1520,49 @@ method(delete)  --> "DELETE", !.
 method(options) --> "OPTIONS", !.
 method(trace)   --> "TRACE", !.
 
+%%	request_uri_parts(+RequestURI, -Parts, ?Tail) is det.
+%
+%	Process the request-uri, producing the following parts:
+%
+%	  * path(-Path)
+%	  Decode path information (always present)
+%	  * search(-QueryParams)
+%	  Present if there is a ?name=value&... part of the request uri.
+%	  QueryParams is a Name=Value list.
+%	  * fragment(-Fragment)
+%	  Present if there is a #Fragment.
+
+request_uri_parts(ReqURI, [path(Path)|Parts], Rest) :-
+	uri_components(ReqURI, Components),
+	uri_data(path, Components, PathText),
+	uri_encoded(path, Path, PathText),
+	phrase(uri_parts(Components), Parts, Rest).
+
+uri_parts(Components) -->
+	uri_search(Components),
+	uri_fragment(Components).
+
+uri_search(Components) -->
+	{ uri_data(search, Components, Search),
+	  nonvar(Search), !,
+	  uri_query_components(Search, Query)
+	},
+	[ search(Query) ].
+uri_search(_) --> [].
+
+uri_fragment(Components) -->
+	{ uri_data(fragment, Components, String),
+	  nonvar(String), !,
+	  uri_encoded(fragment, Fragment, String)
+	},
+	[ fragment(Fragment) ].
+uri_fragment(_) --> [].
+
+%%	request_header(+In:stream, -Header:list) is det.
+%
+%	Read the remainder (after the request-uri)   of  the HTTP header
+%	and return it as a Name(Value) list.
+
 request_header(_, []) -->		% Old-style non-version header
 	blanks,
 	eos, !.
@@ -1314,7 +1612,7 @@ cookie(Name, Value) -->
 
 cookie_name(Name) -->
 	{ var(Name) }, !,
-	rd_field_chars(Chars),
+	rd_field_chars_no_fold(Chars),
 	{ atom_codes(Name, Chars) }.
 
 cookie_value(Value) -->
@@ -1322,32 +1620,29 @@ cookie_value(Value) -->
 	{ atom_codes(Value, Chars)
 	}.
 
-chars_to_semicolon_or_blank([]) -->
-	peek(0';), !.			% 0'
-chars_to_semicolon_or_blank([]) -->
-	blank, !.
 chars_to_semicolon_or_blank([H|T]) -->
-	[H], !,
+	[H],
+	{ H \== 32, H \== 0'; }, !,
 	chars_to_semicolon_or_blank(T).
 chars_to_semicolon_or_blank([]) -->
 	[].
 
-peek(C, L, L) :-
-	L = [C|_].
-
 set_cookie(set_cookie(Name, Value, Options)) -->
-	blanks,
+	ws,
 	cookie(Name, Value),
 	cookie_options(Options).
 
 cookie_options([H|T]) -->
-	blanks,
+	ws,
 	";",
-	blanks,
+	ws,
 	cookie_option(H), !,
 	cookie_options(T).
 cookie_options([]) -->
-	blanks.
+	ws.
+
+ws --> " ", !, ws.
+ws --> [].
 
 
 %%	cookie_option(-Option)// is semidet.
@@ -1361,21 +1656,24 @@ cookie_options([]) -->
 %	@bug	Incorrectly accepts options without = for M$ compatibility.
 
 cookie_option(Name=Value) -->
-	rd_field_chars(NameChars), whites,
+	rd_field_chars(NameChars), ws,
 	{ atom_codes(Name, NameChars) },
 	(   "="
-	->  blanks,
+	->  ws,
 	    chars_to_semicolon(ValueChars),
 	    { atom_codes(Value, ValueChars)
 	    }
 	;   { Value = true }
 	).
 
-chars_to_semicolon([]) -->
-	blanks,
-	peek(0';), !.			% 0'
 chars_to_semicolon([H|T]) -->
-	[H], !,
+	[H],
+	{ H \== 32, H \== 0'; }, !,
+	chars_to_semicolon(T).
+chars_to_semicolon([]), ";" -->
+	ws, ";", !.
+chars_to_semicolon([H|T]) -->
+	[H],
 	chars_to_semicolon(T).
 chars_to_semicolon([]) -->
 	[].
