@@ -153,6 +153,9 @@ type_term(codes,  Result, codes(Result)).
 %		packed string object.  Please note that =codes= or
 %		=chars= would produce ambiguous output and is therefore
 %		not supported.
+%
+%	If json_read/3 encounters end-of-file before any real data it
+%	binds Term to the term @(end_of_file).
 
 json_read(Stream, Term) :-
 	default_json_options(Options),
@@ -166,7 +169,11 @@ json_read(Stream, Term, Options) :-
 json_value(Stream, Term, Next, Options) :-
 	get_code(Stream, C0),
 	ws(C0, Stream, C1),
-	json_term(C1, Stream, Term, Next, Options).
+	(   C1 == -1
+	->  Term = @(end_of_file),
+	    Next = -1
+	;   json_term(C1, Stream, Term, Next, Options)
+	).
 
 json_term(0'{, Stream, json(Pairs), Next, Options) :- !,
 	ws(Stream, C),
