@@ -61,7 +61,6 @@ User extension hooks.
 
 :- multifile
 	style/2,
-	identify/2,
 	term_colours/2,
 	goal_colours/2,
 	goal_classification/2,
@@ -69,6 +68,8 @@ User extension hooks.
 
 :- thread_local
 	fragment/3.			% Start, Length, Class
+:- meta_predicate
+	process_source(+, 3).
 
 :- create_prolog_flag(xref, false, [type(boolean)]).
 
@@ -140,9 +141,9 @@ process_source(Src, Handler) :-
 	;   Xref = false
 	),
 	set_prolog_flag(xref, true),
-	b_setval(doc_colour_handler, Handler),
-	call_cleanup(process_input(Fd, Src),
-		     cleanup_source(Xref, Fd)).
+	setup_call_cleanup(b_setval(doc_colour_handler, Handler),
+			   process_input(Fd, Src),
+			   cleanup_source(Xref, Fd)).
 
 cleanup_source(Xref, Fd) :-
 	set_prolog_flag(xref, Xref),
@@ -576,7 +577,7 @@ colourise_term_arg(Atom, TB, Pos) :-			% atom
 	colour_item(atom, TB, Pos).
 colourise_term_arg(List, TB, list_position(_, _, Elms, Tail)) :- !,
 	colourise_list_args(Elms, Tail, List, TB, classify).	% list
-colourise_term_arg(Compound, TB, Pos) :- 		% compound
+colourise_term_arg(Compound, TB, Pos) :-		% compound
 	compound(Compound), !,
 	colourise_term_args(Compound, TB, Pos).
 colourise_term_arg(_, TB, string_position(F, T)) :- !,	% string
@@ -846,12 +847,12 @@ head_colours(M:_,		    meta-[module(M),extern(M)]).
 
 def_style(goal(built_in,_),	style(colour := blue)).
 def_style(goal(imported(_),_),	style(colour := blue)).
-def_style(goal(autoload,_), 	style(colour := navy_blue)).
+def_style(goal(autoload,_),	style(colour := navy_blue)).
 def_style(goal(global,_),	style(colour := navy_blue)).
 def_style(goal(undefined,_),	style(colour := red)).
 def_style(goal(thread_local(_),_), style(colour := magenta,
 				      underline:= @on)).
-def_style(goal(dynamic(_),_), 	style(colour := magenta)).
+def_style(goal(dynamic(_),_),	style(colour := magenta)).
 def_style(goal(multifile(_),_),	style(colour := navy_blue)).
 def_style(goal(expanded,_),	style(colour := blue,
 				      underline := @on)).
@@ -868,21 +869,21 @@ def_style(head(extern(_)),	style(bold := @on, colour := blue)).
 def_style(head(dynamic),	style(bold := @on, colour := magenta)).
 def_style(head(multifile),	style(bold := @on, colour := navy_blue)).
 def_style(head(unreferenced),	style(bold := @on, colour := red)).
-def_style(head(hook),	  	style(underline  := @on, colour := blue)).
-def_style(head(meta),	  	@default).
+def_style(head(hook),		style(underline  := @on, colour := blue)).
+def_style(head(meta),		@default).
 def_style(head(constraint(_)),	style(bold := @on, colour := darkcyan)).
-def_style(head(_),	  	style(bold := @on)).
+def_style(head(_),		style(bold := @on)).
 
 def_style(comment,		style(colour := dark_green)).
 def_style(structured_comment,	style(colour := dark_green)).
 
-def_style(directive,	  	style(background := grey90)).
-def_style(method(_),	  	style(bold := @on)).
+def_style(directive,		style(background := grey90)).
+def_style(method(_),		style(bold := @on)).
 
-def_style(var,		  	style(colour := red4)).
+def_style(var,			style(colour := red4)).
 def_style(unbound,		style(bold := @on, colour := red)).
 def_style(atom,			@default).		% ensure it passed.
-def_style(quoted_atom,        	style(colour := navy_blue)).
+def_style(quoted_atom,		style(colour := navy_blue)).
 def_style(string,		style(colour := navy_blue)).
 def_style(nofile,		style(colour := red)).
 def_style(file(_),		style(colour := blue,
@@ -893,14 +894,14 @@ def_style(class(library(_),_),	style(colour := navy_blue,
 				      underline := @on)).
 def_style(class(local(_,_,_),_), style(underline := @on)).
 def_style(class(user(_),_),	style(underline := @on)).
-def_style(class(user,_), 	style(underline := @on)).
+def_style(class(user,_),	style(underline := @on)).
 def_style(class(undefined,_),	style(colour := red,
 				      underline  := @on)).
 def_style(prolog_data,		style(colour := blue,
 				      underline  := @on)).
 
-def_style(keyword(_), 		style(colour := blue)).
-def_style(identifier, 		style(bold := @on)).
+def_style(keyword(_),		style(colour := blue)).
+def_style(identifier,		style(bold := @on)).
 def_style(delimiter,		style(bold := @on)).
 def_style(expanded,		style(colour := blue,
 				      underline  := @on)).
@@ -910,10 +911,7 @@ def_style(hook,			style(colour := blue,
 
 def_style(error,		style(background := orange)).
 def_style(type_error(_),	style(background := orange)).
-def_style(syntax_error,	  	style(background := red)).
-
-:- dynamic
-	style_name/2.
+def_style(syntax_error,		style(background := red)).
 
 %%	prolog_src_style(?Class, ?Style) is nondet.
 %
@@ -925,12 +923,6 @@ prolog_src_style(Class, Style) :-
 prolog_src_style(Class, Style) :-
 	def_style(Class, Style).	% system default
 
-style(Class, Name, Style) :-
-	prolog_src_style(Class, Style),
-	copy_term(Class, Copy),
-	numbervars(Copy, 0, _),
-	term_to_atom(Copy, Name).
-
 
 %%	term_colours(+Term, -FunctorColourArgColours)
 %
