@@ -34,7 +34,7 @@
 	  [ doc_for_file/2,		% +FileSpec, +Options
 	    doc_write_html/3,		% +Stream, +Title, +Term
 	    doc_for_wiki_file/2,	% +FileSpec, +Options
-	    				% Support doc_index
+					% Support doc_index
 	    doc_page_dom/3,		% +Title, +Body, -DOM
 	    print_html_head/1,		% +Stream
 	    predref//1,			% +PI //
@@ -99,6 +99,13 @@ extracting module doc_wiki.pl into HTML+CSS.
 	in other backends.
 */
 
+:- public
+	params//1,			% Called from \Term output created
+	pred_dt//3,			% by the wiki renderer
+	section//2,
+	tag//2.
+
+
 		 /*******************************
 		 *	     RESOURCES		*
 		 *******************************/
@@ -1644,7 +1651,18 @@ include(PI, predicate, _) --> !,
 	;   html(['[[', \predref(PI), ']]'])
 	).
 include(File, image, Options) -->
-	{ file_href(File, HREF, Options),
+	{ file_name_extension(_, svg, File),
+	  file_href(File, HREF, Options), !,
+	  include(image_attribute, Options, Attrs0),
+	  merge_options(Attrs0,
+			[ alt(File),
+			  data(HREF),
+			  type('image/svg+xml')
+			], Attrs)
+	},
+	html(object(Attrs, [])).
+include(File, image, Options) -->
+	{ file_href(File, HREF, Options), !,
 	  include(image_attribute, Options, Attrs0),
 	  merge_options(Attrs0,
 			[ alt(File),
