@@ -37,6 +37,7 @@
 	    indented_lines/3,		% +Text, +PrefixChars, -Lines
 	    strip_leading_par/2,	% +DOM0, -DOM
 	    normalise_white_space/3,	% -Text, //
+	    autolink_extension/2,	% ?Extension, ?Type
 	    autolink_file/2		% +FileName, -Type
 	  ]).
 :- use_module(library(lists)).
@@ -436,19 +437,23 @@ renamed_tag(exception, throws).
 %	the tags are forced into  this   order  without  warning. Future
 %	versions may issue a warning if the order is inconsistent.
 
-tag_order(param,       1).
-tag_order(error,       2).		% same as throw
-tag_order(throws,      3).
-tag_order(author,      4).
-tag_order(version,     5).
-tag_order(see,	       6).
-tag_order(deprecated,  7).
-tag_order(compat,      8).		% PlDoc extension
-tag_order(copyright,   9).
-tag_order(license,    10).
-tag_order(bug,	      11).
-tag_order(tbd,	      12).
+:- multifile
+	pldoc:tag_order/2.
 
+tag_order(Tag, Order) :-
+	pldoc:tag_order(Tag, Order), !.
+tag_order(param,       100).
+tag_order(error,       200).		% same as throw
+tag_order(throws,      300).
+tag_order(author,      400).
+tag_order(version,     500).
+tag_order(see,	       600).
+tag_order(deprecated,  700).
+tag_order(compat,      800).		% PlDoc extension
+tag_order(copyright,   900).
+tag_order(license,    1000).
+tag_order(bug,	      1100).
+tag_order(tbd,	      1200).
 
 %%	combine_tags(+Tags:list(tag(Key, Value)), -Tags:list) is det.
 %
@@ -943,6 +948,7 @@ autolink_extension(gif, image).
 autolink_extension(png, image).
 autolink_extension(jpg, image).
 autolink_extension(jpeg, image).
+autolink_extension(svg, image).
 
 %%	autolink_file(?File, -Type) is nondet.
 %
@@ -1310,32 +1316,6 @@ strip_leading_par([p(C)|T], L) :- !,
 strip_leading_par(L, L).
 
 
-%%	bind_vars(+Term, +Bindings) is det.
-
-bind_vars(Term, Bindings) :-
-	bind_vars(Bindings),
-	anon_vars(Term).
-
-bind_vars([]).
-bind_vars([Name=Var|T]) :-
-	Var = '$VAR'(Name),
-	bind_vars(T).
-
-%%	anon_vars(+Term) is det.
-%
-%	Bind remaining variables in Term to '$VAR'('_'), so they are
-%	printed as '_'.
-
-anon_vars(Var) :-
-	var(Var), !,
-	Var = '$VAR'('_').
-anon_vars(Term) :-
-	compound(Term), !,
-	Term =.. [_|Args],
-	maplist(anon_vars, Args).
-anon_vars(_).
-
-
 		 /*******************************
 		 *	     DCG BASICS		*
 		 *******************************/
