@@ -72,9 +72,9 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 	debug(clause_info, 'from ~w:~d ... ', [File, LineNo]),
 	read_term_at_line(File, LineNo, Module, Clause, TermPos0, VarNames),
 	debug(clause_info, 'read ...', []),
-	unify_clause(Clause, DecompiledClause, TermPos0, TermPos),
+	unify_clause(Clause, DecompiledClause, Module, TermPos0, TermPos),
 	debug(clause_info, 'unified ...', []),
-	make_varnames(Clause, VarOffset, VarNames, NameOffset),
+	make_varnames(Clause, DecompiledClause, VarOffset, VarNames, NameOffset),
 	debug(clause_info, 'got names~n', []), !.
 
 %%	unify_term(+T1, +T2)
@@ -88,6 +88,9 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 %	NOTE: Called directly from  library(trace/clause)   for  the GUI
 %	tracer.
 
+:- public
+	unify_term/2.
+
 unify_term(X, X) :- !.
 unify_term(X1, X2) :-
 	compound(X1),
@@ -101,6 +104,8 @@ unify_term(X, Y) :-
 	string(X),
 	is_list(Y),
 	string_to_list(X, Y), !.
+unify_term(_, Y) :-
+	Y == '...', !.				% elipses left by max_depth
 unify_term(_:X, Y) :-
 	unify_term(X, Y), !.
 unify_term(X, _:Y) :-
@@ -178,18 +183,32 @@ read(Handle, Module, Clause, TermPos, VarNames) :-
 	).
 
 
-%%	make_varnames(+ReadClause, +Offsets, +Names, -Term) is det.
+%%	make_varnames(+ReadClause, +DecompiledClause,
+%%		      +Offsets, +Names, -Term) is det.
 %
 %	Create a Term varnames(...) where each argument contains the name
 %	of the variable at that offset.  If the read Clause is a DCG rule,
 %	name the two last arguments <DCG_list> and <DCG_tail>
 %
+%	This    predicate    calles     the      multifile     predicate
+%	make_varnames_hook/5 with the same arguments   to allow for user
+%	extensions. Extending this predicate  is   needed  if a compiler
+%	adds additional arguments to the clause   head that must be made
+%	visible in the GUI tracer.
+%
 %	@param Offsets	List of Offset=Var
 %	@param Names	List of Name=Var
 %
 %	@bug Called directly from library(trace/clause) for the GUI tracer.
 
-make_varnames((Head --> _Body), Offsets, Names, Bindings) :- !,
+:- multifile make_varnames_hook/5.
+:- public
+	make_varnames/5,
+	do_make_varnames/3.		% allow usage from the hook.
+
+make_varnames(ReadClause, DecompiledClause, Offsets, Names, Term) :-
+	make_varnames_hook(ReadClause, DecompiledClause, Offsets, Names, Term), !.
+make_varnames((Head --> _Body), _, Offsets, Names, Bindings) :- !,
 	functor(Head, _, Arity),
 	In is Arity,
 	memberchk(In=IVar, Offsets),
@@ -197,8 +216,8 @@ make_varnames((Head --> _Body), Offsets, Names, Bindings) :- !,
 	Out is Arity + 1,
 	memberchk(Out=OVar, Offsets),
 	Names2 = ['<DCG_tail>'=OVar|Names1],
-	make_varnames(xx, Offsets, Names2, Bindings).
-make_varnames(_, Offsets, Names, Bindings) :-
+	make_varnames(xx, xx, Offsets, Names2, Bindings).
+make_varnames(_, _, Offsets, Names, Bindings) :-
 	length(Offsets, L),
 	functor(Bindings, varnames, L),
 	do_make_varnames(Offsets, Names, Bindings).
@@ -218,26 +237,35 @@ find_varname(Var, [Name = TheVar|_], Name) :-
 find_varname(Var, [_|T], Name) :-
 	find_varname(Var, T, Name).
 
-%%	unify_clause(+Read, +Decompiled, +ReadTermPos, -RecompiledTermPos).
+%%	unify_clause(+Read, +Decompiled, +Module, +ReadTermPos,
+%%		     -RecompiledTermPos).
 %
 %	What you read isn't always what goes into the database. The task
 %	of this predicate is to establish  the relation between the term
 %	read from the file and the result from decompiling the clause.
 %
-%	This really must be  more  flexible,   dealing  with  much  more
-%	complex source-translations, falling back to  a heuristic method
-%	locating as much as possible.
+%	This predicate calls the multifile predicate unify_clause_hook/5
+%	with the same arguments to support user extensions.
+%
+%	@tbd	This really must be  more   flexible,  dealing with much
+%		more complex source-translations,  falling   back  to  a
+%		heuristic method locating as much as possible.
 
-unify_clause(Read, Read, TermPos, TermPos) :- !.
+:- multifile
+	unify_clause_hook/5.
+
+unify_clause(Read, Read, _, TermPos, TermPos) :- !.
 					% XPCE send-methods
-unify_clause(:->(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
+unify_clause(Read, Decompiled, Module, TermPoso, TermPos) :-
+	unify_clause_hook(Read, Decompiled, Module, TermPoso, TermPos), !.
+unify_clause(:->(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
 	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
 					% XPCE get-methods
-unify_clause(:<-(Head, Body), (PlHead :- PlBody), TermPos0, TermPos) :- !,
+unify_clause(:<-(Head, Body), (PlHead :- PlBody), _, TermPos0, TermPos) :- !,
 	pce_method_clause(Head, Body, PlHead, PlBody, TermPos0, TermPos).
 					% Unit test clauses
 unify_clause((TH :- Body),
-	     (_:'unit body'(_, _) :- !, Body),
+	     (_:'unit body'(_, _) :- !, Body), _,
 	     TP0, TP) :-
 	(   TH = test(_,_)
 	;   TH = test(_)
@@ -246,17 +274,17 @@ unify_clause((TH :- Body),
 	TP  = term_position(F,T,FF,FT,[HP,term_position(0,0,0,0,[FF-FT,BP])]).
 					% module:head :- body
 unify_clause((Head :- Read),
-	     (Head :- _M:Compiled), TermPos0, TermPos) :-
-	unify_clause((Head :- Read), (Head :- Compiled), TermPos0, TermPos1),
+	     (Head :- _M:Compiled), Module, TermPos0, TermPos) :-
+	unify_clause((Head :- Read), (Head :- Compiled), Module, TermPos0, TermPos1),
 	TermPos1 = term_position(TA,TZ,FA,FZ,[PH,PB]),
 	TermPos  = term_position(TA,TZ,FA,FZ,
 				 [ PH,
 				   term_position(0,0,0,0,[0-0,PB])
 				 ]).
-unify_clause(Read, Compiled1, TermPos0, TermPos) :-
+unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	Read = (_ --> List, _),
 	is_list(List),
-	catch(expand_term(Read, Compiled2), E, expand_failed(E, Read)),
+	ci_expand(Read, Compiled2, Module),
 	Compiled2 = (DH :- _),
 	functor(DH, _, Arity),
 	DArg is Arity - 1,
@@ -268,11 +296,11 @@ unify_clause(Read, Compiled1, TermPos0, TermPos) :-
 	TermPos1 = term_position(F,T,FF,FT,[ HP, BP ]),
 	match_module(Compiled2, Compiled1, TermPos1, TermPos).
 					% general term-expansion
-unify_clause(Read, Compiled1, TermPos0, TermPos) :-
-	catch(expand_term(Read, Compiled2), E, expand_failed(E, Read)),
+unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
+	ci_expand(Read, Compiled2, Module),
 	match_module(Compiled2, Compiled1, TermPos0, TermPos).
 					% I don't know ...
-unify_clause(_, _, _, _) :-
+unify_clause(_, _, _, _, _) :-
 	debug(clause_info, 'Could not unify clause', []),
 	fail.
 
@@ -280,6 +308,13 @@ unify_clause_head(H1, H2) :-
 	strip_module(H1, _, H),
 	strip_module(H2, _, H).
 
+ci_expand(Read, Compiled, Module) :-
+	catch(setup_call_cleanup('$set_source_module'(Old, Module),
+				 expand_term(Read, Compiled),
+				 '$set_source_module'(_, Old)),
+	      E,
+	      expand_failed(E, Read)).
+
 match_module((H1 :- B1), (H2 :- B2), Pos0, Pos) :- !,
 	unify_clause_head(H1, H2),
 	unify_body(B1, B2, Pos0, Pos).
