@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        J.Wielemaker@uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2009, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -35,9 +36,18 @@
 	  ]).
 :- use_module(library(lists)).
 
-:- module_transparent
-	qsave_program/1,
-	qsave_program/2.
+/** <module> Save current program as a state or executable
+
+This library provides qsave_program/1  and   qsave_program/2,  which are
+also used by the commandline sequence below.
+
+  ==
+  swipl -o exe -c file.pl ...
+  ==
+*/
+
+:- meta_predicate
+	qsave_program(+, :).
 
 :- system_mode(on).
 
@@ -45,16 +55,15 @@
 :- volatile verbose/1.			% contains a stream-handle
 
 %%	qsave_program(+File) is det.
-%%	qsave_program(+File, +Options) is det.
+%%	qsave_program(+File, :Options) is det.
 %
 %	Make a saved state in file `File'.
 
 qsave_program(File) :-
 	qsave_program(File, []).
 
-qsave_program(FileSpec, Options0) :-
+qsave_program(FileBase, Module:Options0) :-
 	check_options(Options0),
-	strip_module(FileSpec, Module, FileBase),
 	exe_file(FileBase, File),
 	option(Options0, autoload/true,	    Autoload,  Options1),
 	option(Options1, map/[],	    Map,       Options2),
@@ -100,7 +109,6 @@ qsave_program(FileSpec, Options0) :-
 	save_prolog_flags,
 	save_operators(SaveOps),
 	save_format_predicates,
-	save_functions,
 	system_mode(off),
 	'$close_wic',
 	close(StateFd),
@@ -116,7 +124,7 @@ exe_file(Exe, Exe).
 
 default_init_file(runtime, none) :- !.
 default_init_file(_,       InitFile) :-
-	'$option'(init_file, InitFile, InitFile).
+	'$option'(init_file, InitFile).
 
 
 		 /*******************************
@@ -159,7 +167,6 @@ make_header(_, _, _).
 min_stack(local,    32).
 min_stack(global,   16).
 min_stack(trail,    16).
-min_stack(argument, 16).
 
 convert_option(Stack, Val, NewVal) :-	% stack-sizes are in K-bytes
 	min_stack(Stack, Min), !,
@@ -169,6 +176,14 @@ convert_option(Stack, Val, NewVal) :-	% stack-sizes are in K-bytes
 	).
 convert_option(_, Val, Val).
 
+doption(Name) :- min_stack(Name, _).
+doption(goal).
+doption(toplevel).
+doption(init_file).
+doption(system_init_file).
+doption(class).
+doption(home).
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Save the options in the '$options' resource.   The home directory is saved
 for development saves, so it keeps refering to the development home.
@@ -179,12 +194,12 @@ avoid a save-script loading itself.
 
 save_options(RC, Options) :-
 	'$rc_open'(RC, '$options', '$prolog', write, Fd),
-	(   '$option'(OptionName, OptionVal0, _),
+	(   doption(OptionName),
+	    '$option'(OptionName, OptionVal0),
 	        (   OptionName == home	% save home if not runtime
 		->  \+ memberchk(class(runtime), Options)
 		;   true
 		),
-	        \+ OptionName == script_file,
 	        option(Options, OptionName/_, OptionVal1, _),
 	        (   var(OptionVal1)	% used the default
 		->  OptionVal = OptionVal0
@@ -338,6 +353,7 @@ attrib_name(dynamic,	   dynamic,	  1).
 attrib_name(volatile,	   volatile,	  1).
 attrib_name(thread_local,  thread_local,  1).
 attrib_name(multifile,	   multifile,	  1).
+attrib_name(public,	   public,	  1).
 attrib_name(transparent,   transparent,	  1).
 attrib_name(discontiguous, discontiguous, 1).
 attrib_name(notrace,	   trace,	  0).
@@ -412,16 +428,13 @@ save_flags :-
 		 *	     IMPORTS		*
 		 *******************************/
 
-default_import(system, _, _) :- !, fail.
-default_import(To, Head, _) :-
-	'$get_predicate_attribute'(To:Head, (dynamic), 1), !,
-	fail.
-default_import(user, Head, _) :- !,
-	'$default_predicate'(user:Head, system:Head).
-default_import(To, Head, _From) :-
-	'$default_predicate'(To:Head, user:Head).
-default_import(To, Head, _From) :-
-	'$default_predicate'(To:Head, system:Head).
+%%	save_imports
+%
+%	Save  import  relations.  An  import  relation  is  saved  if  a
+%	predicate is imported from a module that is not a default module
+%	for the destination module. If  the   predicate  is  dynamic, we
+%	always define the explicit import relation to make clear that an
+%	assert must assert on the imported predicate.
 
 save_imports :-
 	feedback('~nIMPORTS~n~n', []),
@@ -434,6 +447,13 @@ save_imports :-
 	;   true
 	).
 
+default_import(To, Head, From) :-
+	'$get_predicate_attribute'(To:Head, (dynamic), 1),
+	predicate_property(From:Head, exported), !,
+	fail.
+default_import(Into, _, From) :-
+	default_module(Into, From).
+
 %%	restore_import(+TargetModule, +SourceModule, +PI) is det.
 %
 %	Restore import relation. This notably   deals  with imports from
@@ -521,24 +541,6 @@ qualify_head(T, T) :-
 qualify_head(T, user:T).
 
 
-		 /*******************************
-		 *	     FUNCTIONS		*
-		 *******************************/
-
-save_functions :-
-	feedback('~nFUNCTIONS~n', []),
-	'$prolog_arithmetic_function'(M:Head, Index),
-	functor(Head, Name, Arity),
-	PredArity is Arity + 1,
-	functor(PredHead, Name, PredArity),
-	D = '$arithmetic_function'(M:PredHead, Index),
-	feedback('~n~t~8|~w ', [D]),
-	'$add_directive_wic'(D),
-	fail.
-save_functions.
-
-
-
 		 /*******************************
 		 *	       UTIL		*
 		 *******************************/
@@ -577,10 +579,10 @@ option_type(autoload,	 bool).
 option_type(map,	 atom).
 option_type(op,		 atom([save, standard])).
 option_type(stand_alone, bool).
-option_type(goal, 	 callable).
-option_type(toplevel, 	 callable).
-option_type(init_file, 	 atom).
-option_type(emulator, 	 ground).
+option_type(goal,	 callable).
+option_type(toplevel,	 callable).
+option_type(init_file,	 atom).
+option_type(emulator,	 ground).
 
 check_options([]) :- !.
 check_options([Var|_]) :-
