@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2004-2007, University of Amsterdam
+    Copyright (C): 2004-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -31,104 +32,221 @@
 
 :- module(rdf_turtle,
 	  [ rdf_load_turtle/3,		% +Input, -Triples, +Options
-	    rdf_process_turtle/3	% +Input, :ObObject, +Options
+	    rdf_read_turtle/3,		% +Input, -Triples, +Options
+	    rdf_process_turtle/3	% +Input, :OnObject, +Options
 	  ]).
 :- use_module(library(assoc)).
 :- use_module(library(option)).
 :- use_module(library('semweb/rdf_db')).
 :- use_module(library(debug)).
-:- use_module(library(url)).
+:- use_module(library(uri)).
+:- use_module(library(record)).
 :- use_module(library(http/http_open)).
+:- use_module(turtle_base).
 
 :- meta_predicate
-	rdf_process_turtle(+,:,+).
+	rdf_process_turtle(+,2,+).
 
-/** <module> Turtle - Terse RDF Triple Language
+/** <module> Turtle: Terse RDF Triple Language
 
 This module implements the Turtle  language   for  representing  the RDF
 triple model as defined by Dave Beckett  from the Institute for Learning
 and Research Technology University of Bristol in the document:
 
-	* http://www.ilrt.bris.ac.uk/discovery/2004/01/turtle/
+  * http://www.w3.org/TeamSubmission/turtle/
+  * http://www.w3.org/TeamSubmission/2008/SUBM-turtle-20080114/#sec-conformance
 
-The current parser handles all positive   and negative examples provided
-by the above document at october 17, 2004.
+This parser passes all tests,  except   for  test-28.ttl  (decial number
+serialization) and test-29.ttl (uri containing  ...%&...). It is unclear
+to me whether these tests are correct. Notably, it is unclear whether we
+must do %-decoding. Certainly, this  is   expected  by various real-life
+datasets that we came accross with.
 
-@tbd	* Much better error handling
-	* Write turtle data
+This module acts as a plugin to   rdf_load/2,  for processing files with
+one of the extensions =|.ttl|=, =|.n3|= or =|.nt|=.
+
+@tbd Better error handling
 */
 
-%%	rdf_load_turtle(+Input, -Triples, +Options)
-%	
+:- record ttl_state(base_uri,
+		    resources:oneof([uri,iri])=uri,
+		    prefix_map,
+		    nodeid_map,
+		    anon_prefix,
+		    anon_count=0,
+		    graph,
+		    input,
+		    line_no=0,
+		    on_error:oneof([warning,error])=warning,
+		    error_count=0).
+
+%%	rdf_read_turtle(+Input, -Triples, +Options)
+%
 %	Read a stream or file into a set of triples of the format
-%	
+%
 %		rdf(Subject, Predicate, Object)
-%		
+%
 %	The representation is consistent with the SWI-Prolog RDF/XML
 %	and ntriples parsers.  Provided options are:
-%	
+%
 %		* base_uri(+BaseURI)
 %		Initial base URI.  Defaults to file://<file> for loading
 %		files.
-%		
+%
 %		* anon_prefix(+Prefix)
 %		Blank nodes are generated as <Prefix>1, <Prefix>2, etc.
 %		If Prefix is not an atom blank nodes are generated as
 %		node(1), node(2), ...
+%
+%		* resources(URIorIRI)
+%		Officially, Turtle resources are IRIs.  Quite a
+%		few applications however send URIs.  By default we
+%		do URI->IRI mapping because this rarely causes errors.
+%		To force strictly conforming mode, pass =iri=.
+%
+%		* prefixes(-Pairs)
+%		Return encountered prefix declarations as a
+%		list of Alias-URI
+%
+%		* namespaces(-Pairs)
+%		Same as prefixes(Pairs).  Compatibility to rdf_load/2.
+%
+%		* base_used(-Base)
+%		Base URI used for processing the data.  Unified to
+%		[] if there is no base-uri.
+%
+%		* on_error(+ErrorMode)
+%		In =warning= (default), print the error and continue
+%		parsing the remainder of the file.  If =error=, abort
+%		with an exception on the first error encountered.
+%
+%		* error_count(-Count)
+%		If on_error(warning) is active, this option cane be
+%		used to retrieve the number of generated errors.
 
-rdf_load_turtle(In, Triples, Options) :-
-	open_input(In, Stream),
+rdf_read_turtle(In, Triples, Options) :-
+	open_input(In, Stream, Close),
 	init_state(In, Stream, Options, State),
 	call_cleanup(phrase(turtle_file(State, Stream), Triples),
-		     close(Stream)).
+		     Close),
+	post_options(State, Options).
+
+
+%%	rdf_load_turtle(+Input, -Triples, +Options)
+%
+%	@deprecated Use rdf_read_turtle/3
+
+rdf_load_turtle(Input, Triples, Options) :-
+	rdf_read_turtle(Input, Triples, Options).
 
 
 %%	rdf_process_turtle(+Input, :OnObject, +Options) is det.
 %
-%	Process Turtle input from Input, calling OnObject with a list
-%	of triples.  Options is the same as for rdf_load_turtle/3.
+%	Process Turtle input from Input, calling OnObject with a list of
+%	triples. Options is the same as for rdf_load_turtle/3.
+%
+%	Errors encountered are sent to  print_message/2, after which the
+%	parser tries to recover and parse the remainder of the data.
 
 rdf_process_turtle(In, OnObject, Options) :-
-	strip_module(OnObject, M, G),
-	open_input(In, Stream),
+	open_input(In, Stream, Close),
 	init_state(In, Stream, Options, State),
-	process_stream(State, Stream, M:G).
+	call_cleanup(process_stream(State, Stream, OnObject),
+		     Close),
+	post_options(State, Options).
 
+post_options(State, Options) :-
+	prefix_option(State, Options),
+	namespace_option(State, Options),
+	base_option(State, Options),
+	error_option(State, Options).
 
-process_stream(State, In, OnObject) :-
-	line_count(In, LineNo),
-	nb_setarg(8, State, LineNo),
-	(   turtle_tokens(In, Tokens)
-	->  debug(turtle, 'Tokens: ~w~n', [Tokens])
-	;   syntax_error(In, LineNo, illegal_token)
-	),
-	(   Tokens == end_of_file
-	->  true
-	;   phrase(triples(State, Triples), Tokens)
-	->  (   Triples == []
-	    ->	true
-	    ;	arg(6, State, DB),
-		call(OnObject, Triples, DB:LineNo)
-	    ),
-	    process_stream(State, In, OnObject)
-	;   syntax_error(In, LineNo, cannot_parse)
+prefix_option(State, Options) :-
+	(   option(prefixes(Pairs), Options)
+	->  ttl_state_prefix_map(State, Map),
+	    assoc_to_list(Map, Pairs)
+	;   true
+	).
+namespace_option(State, Options) :-
+	(   option(namespaces(Pairs), Options)
+	->  ttl_state_prefix_map(State, Map),
+	    assoc_to_list(Map, Pairs)
+	;   true
+	).
+base_option(State, Options) :-
+	(   option(base_used(Base), Options)
+	->  ttl_state_base_uri(State, Base)
+	;   true
+	).
+error_option(State, Options) :-
+	(   option(error_count(Count), Options)
+	->  ttl_state_error_count(State, Count)
+	;   true
 	).
 
 
-%%	open_input(+Input, -Stream) is det.
+process_stream(State, In, OnObject) :-
+	read_turtle_tokens(In, Tokens, State),
+	debug(turtle, 'Tokens: ~w~n', [Tokens]),
+	ttl_state_line_no(State, LineNo),
+	(   Tokens == end_of_file
+	->  true
+	;   catch(phrase(triples(State, Triples), Tokens), E, true)
+	->  (   var(E)
+	    ->  (   Triples == []
+		->  true
+		;   ttl_state_graph(State, DB),
+		    call(OnObject, Triples, DB:LineNo)
+		)
+	    ;   print_message(error, E)
+	    ),
+	    process_stream(State, In, OnObject)
+	;   syntax_error_term(In, LineNo, cannot_parse, Error),
+	    step_error(State, Error),
+	    process_stream(State, In, OnObject)
+	).
+
+
+%%	step_error(+State, +Error) is det.
+%
+%	Throw Error of =on_error= is =error=.  Otherwise print the error
+%	and increment =error_count=.
+%
+%	@error syntax_error(Culprit).
+
+step_error(State, Error) :-
+	ttl_state_on_error(State, error), !,
+	throw(Error).
+step_error(State, Error) :-
+	ttl_state_error_count(State, E0),
+	succ(E0, E),
+	nb_set_error_count_of_ttl_state(E, State),
+	print_message(error, Error).
+
+
+%%	open_input(+Input, -Stream, -Close) is det.
 %
 %	Open given input.
 %
+%	@param  Close goal to undo the open action
 %	@tbd	Synchronize with input handling of rdf_db.pl.
 %	@error	existence_error, permission_error
 
-open_input(stream(Stream), Stream) :- !.
-open_input(Stream, Stream) :-
-	is_stream(Stream), !.
-open_input(URL, Stream) :-
+open_input(stream(Stream), Stream, true) :- !,
+	stream_property(Stream, encoding(Old)),
+	(   Old == utf8
+	->  Close = true
+	;   set_stream(Stream, encoding(utf8)),
+	    Close = set_stream(Stream, encoding(Old))
+	).
+open_input(Stream, Stream, Close) :-
+	is_stream(Stream), !,
+	open_input(stream(Stream), Stream, Close).
+open_input(URL, Stream, close(Stream)) :-
 	sub_atom(URL, 0, _, _, 'http://'), !,
-	http_open(URL, Stream, []).
-open_input(File, Stream) :-
+	http_open(URL, Stream, []),
+	set_stream(Stream, encoding(utf8)).
+open_input(File, Stream, close(Stream)) :-
 	absolute_file_name(File, Path,
 			   [ access(read),
 			     extensions([ttl, ''])
@@ -140,16 +258,7 @@ The parser is a two-stage processor. The  first reads the raw file input
 and generates a list of tokens, stripping   comments and white space. It
 is defined to read a single  statement   upto  its  terminating '.'. The
 second stage is a traditional DCG parser  generating the triples for the
-statement. State:
-
-	| arg(1) | -BaseURI		|
-	| arg(2) | Prefix --> URI map	|
-	| arg(3) | NodeID --> URI map	|
-	| arg(4) | AnonPrefix		|
-	| arg(5) | AnonCount		|
-	| arg(6) | DB			|
-	| arg(7) | Input		|
-	| arg(8) | StartLine		|
+statement.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 init_state(In, Stream, Options, State) :-
@@ -157,10 +266,10 @@ init_state(In, Stream, Options, State) :-
 	->  true
 	;   In = stream(_)
 	->  BaseURI = []
-	;   is_absolute_url(In)
-	->  BaseURI = In
-	;   absolute_file_name(In, File),
-	    atom_concat('file://', File, BaseURI)
+	;   uri_is_global(In),
+	    \+ is_absolute_file_name(In)	% Avoid C:Path in Windows
+	->  uri_normalized(In, BaseURI)
+	;   uri_file_name(BaseURI, In)
 	),
 	(   option(anon_prefix(Prefix), Options)
 	->  true
@@ -169,59 +278,80 @@ init_state(In, Stream, Options, State) :-
 	;   atom_concat('__', BaseURI, Prefix)
 	),
 	option(db(DB), Options, BaseURI),
+	option(on_error(OnError), Options, warning),
+	option(resources(URIIRI), Options, uri),
 	empty_assoc(Map),
 	empty_assoc(NodeMap),
-	State = state(BaseURI, Map, NodeMap, Prefix, 1, DB, Stream, 0).
+	make_ttl_state([ base_uri(BaseURI),
+			 resources(URIIRI),
+			 prefix_map(Map),
+			 nodeid_map(NodeMap),
+			 anon_prefix(Prefix),
+			 graph(DB),
+			 input(Stream),
+			 on_error(OnError)
+		       ], State).
 
 
 turtle_file(State, In) -->
-	{ line_count(In, LineNo),
-	  nb_setarg(8, State, LineNo),
-	  (   turtle_tokens(In, Tokens)
-	  ->  debug(turtle, 'Tokens: ~w~n', [Tokens])
-	  ;   syntax_error(In, LineNo, illegal_token)
-	  )
+	{ read_turtle_tokens(In, Tokens, State),
+	  debug(turtle, 'Tokens: ~w~n', [Tokens])
 	},
 	(   { Tokens == end_of_file }
 	->  []
-	;   { phrase(triples(State, Triples), Tokens) }
-	->  Triples,
+	;   { catch(phrase(triples(State, Triples), Tokens), E, true) }
+	->  (   { var(E) }
+	    ->	list(Triples),
+		turtle_file(State, In)
+	    ;	{ step_error(State, E) },
+		turtle_file(State, In)
+	    )
+	;   { ttl_state_line_no(State, LineNo),
+	      syntax_error_term(In, LineNo, cannot_parse, Error),
+	      step_error(State, Error)
+	    },
 	    turtle_file(State, In)
-	;   { syntax_error(In, LineNo, cannot_parse)
-	    }
 	).
 
+list([]) --> [].
+list([H|T]) --> [H], list(T).
+
 triples(State, []) -->
 	[ '@', name(prefix), name(Prefix), : ], !,
-	uri(State, URI),
-	{ arg(2, State, Map0),
+	iri(State, URI),
+	{ ttl_state_prefix_map(State, Map0),
 	  put_assoc(Prefix, Map0, URI, Map),
-	  setarg(2, State, Map)
+	  set_prefix_map_of_ttl_state(Map, State)
 	}.
 triples(State, []) -->
 	[ '@', name(prefix), ':' ], !,
-	uri(State, URI),
-	{ setarg(1, State, URI)
+	iri(State, URI),
+	{ set_base_uri_of_ttl_state(URI, State)
 	}.
 triples(State, []) -->
-      ['@',name(base)],!,
-      uri(State,URI),
-      { setarg(1,State,URI)
-      }.
+	[ '@', name(base) ], !,
+	iri(State,URI),
+	{ set_base_uri_of_ttl_state(URI, State)
+	}.
 triples(State, Triples) -->
 	subject(State, Subject, Triples, T),
 	(   predicate_object_list(State, Subject, T, [])
-	->  []
+	->  (   eos
+	    ->	[]
+	    ;	syntax_rule(State, expected(predicate_object_list))
+	    )
 	;   { Triples \== T }		% [ p o ; ... ] .
 	->  { T = [] }
 	).
-	
+
+eos([], []).
+
 subject(State, Subject, T, T) -->
 	resource(State, Subject), !.
 subject(State, Subject, T0, T) -->
 	blank(State, Subject, T0, T), !.
 subject(State, _, _, _) -->
-	syntax_error(State, subject_expected).
+	syntax_rule(State, subject_expected).
 
 predicate_object_list(State, Subject, Triples, Tail) -->
 	verb(State, Predicate),
@@ -258,14 +388,34 @@ object(State, Object, T, T) -->
 object(_, literal(type(Type, N)), T, T) -->
 	[ numeric(Tp, Codes) ], !,
 	{ numeric_url(Tp, Type),
-	  atom_codes(N, Codes)
+	  normalise_number(Tp, Codes, N)
 	}.
 object(State, Object, T, T) -->
 	resource(State, Object), !.
 object(State, Object, T0, T) -->
 	blank(State, Object, T0, T), !.
+object(_, Object, T, T) -->
+	[ name(Bool) ],
+	{ boolean(Bool),
+	  Object = literal(type(BoolType, Bool)),
+	  rdf_equal(BoolType, xsd:boolean)
+	}.
 object(State, _, _, _) -->
-	syntax_error(State, expected_object).
+	syntax_rule(State, expected_object).
+
+%%	normalise_number(+Type, +Codes:list, -Literal:atom) is det.
+%
+%	Turtle normalisation of numbers. Currently  only implemented for
+%	integers. This ensures that 0001 is parsed as "1"^^xsd:integer.
+%
+%	Hmmm.  Acording to test-10.ttl, this must *not* be done, so for
+%	now we disable all normalization.
+
+%normalise_number(integer, Codes, N) :-
+%	number_codes(I, Codes),
+%	atom_number(N, I).
+normalise_number(_, Codes, N) :-
+	atom_codes(N, Codes).
 
 term_expansion(numeric_url(I, Local),
 	       numeric_url(I, URI)) :-
@@ -275,50 +425,60 @@ numeric_url(integer, xsd:integer).
 numeric_url(decimal, xsd:decimal).
 numeric_url(double,  xsd:double).
 
-resource(State, URI) -->
-	uri(State, URI), !.
-resource(State, URI) -->
+boolean(true).
+boolean(false).
+
+resource(State, IRI) -->
+	iri(State, IRI), !.
+resource(State, IRI) -->
 	[ :(Name) ], !,
-	{ arg(1, State, Base),
-	  atom_concat(Base, Name, URI)
+	{ ttl_state_base_uri(State, Base),
+	  atom_concat(Base, Name, URI),
+	  uri_iri(State, URI, IRI)
 	}.
-resource(State, URI) -->
+resource(State, IRI) -->
 	[ name(Prefix), : ], !,
-	{ arg(2, State, Map),
-	  get_assoc(Prefix, Map, URI)
+	{ ttl_state_prefix_map(State, Map),
+	  get_assoc(Prefix, Map, IRI)
 	}.
-resource(State, URI) -->
+resource(State, IRI) -->
 	[ Prefix:Name ], !,
-	{ arg(2, State, Map),
+	{ ttl_state_prefix_map(State, Map),
 	  (   get_assoc(Prefix, Map, Base)
-	  ->  atom_concat(Base, Name, URI)
+	  ->  atom_concat(Base, Name, URI),
+	      uri_iri(State, URI, IRI)
 	  ;   throw(error(existence_error(prefix, Prefix), _))
 	  )
 	}.
-resource(State, BaseURI) -->
+resource(State, BaseIRI) -->
 	[ : ], !,
-	{ arg(1, State, BaseURI)
+	{ ttl_state_base_uri(State, BaseIRI)
 	}.
-	
 
-uri(State, URI) -->
+uri_iri(State, URI, IRI) :-
+	(   ttl_state_resources(State, uri)
+	->  uri_iri(URI, IRI)
+	;   IRI = URI
+	).
+
+iri(State, IRI) -->
 	[ relative_uri(Rel)
 	],
-	{ arg(1, State, Base),
+	{ ttl_state_base_uri(State, Base),
 	  (   Rel == ''			% must be in global_url?
-	  ->  URI = Base
-	  ;   global_url(Rel, Base, URI)
+	  ->  IRI = Base
+	  ;   uri_normalized_iri(Rel, Base, IRI)
 	  )
 	}.
 
 blank(State, Resource, T, T) -->
 	[ nodeId(NodeId) ], !,
-	{ arg(3, State, IdMap),
+	{ ttl_state_nodeid_map(State, IdMap),
 	  (   get_assoc(NodeId, IdMap, Resource)
 	  ->  true
 	  ;   anonid(State, NodeId, Resource),
 	      put_assoc(NodeId, IdMap, Resource, NewIdMap),
-	      setarg(3, State, NewIdMap)
+	      set_nodeid_map_of_ttl_state(NewIdMap, State)
 	  )
 	}.
 blank(State, Resource, T, T) -->
@@ -353,54 +513,147 @@ item_list(State, Resource, T0, T) -->
 
 
 anonid(State, Node) :-
-	arg(4, State, AnonPrefix),
-	arg(5, State, Count),
+	ttl_state_anon_prefix(State, AnonPrefix),
+	ttl_state_anon_count(State, C0),
+	Count is C0 + 1,
+	set_anon_count_of_ttl_state(Count, State),
 	(   atom(AnonPrefix)
 	->  atom_concat(AnonPrefix, Count, Node)
 	;   Node = node(Count)
-	),
-	C2 is Count + 1,
-	setarg(5, State, C2).
+	).
 
 anonid(State, _NodeId, Node) :-
-	arg(4, State, AnonPrefix),
+	ttl_state_anon_prefix(State, AnonPrefix),
 	atom(AnonPrefix), !,
 	anonid(State, Node).
 anonid(_State, NodeId, node(NodeId)).
 
 mk_object(type(Prefix:Name, Value), State, literal(type(Type, Value))) :- !,
-	  arg(2, State, Map),
+	  ttl_state_prefix_map(State, Map),
 	  get_assoc(Prefix, Map, Base),
 	  atom_concat(Base, Name, Type).
 mk_object(type(relative_uri(Rel), Value), State, literal(type(Type, Value))) :- !,
-	  arg(1, State, Base),
+	  ttl_state_base_uri(State, Base),
 	  (   Rel == ''			% must be in global_url?
 	  ->  Type = Base
-	  ;   global_url(Rel, Base, Type)
+	  ;   uri_normalized_iri(Rel, Base, Type)
 	  ).
 mk_object(type(:(Name), Value), State, literal(type(Type, Value))) :- !,
-	  arg(1, State, Base),
+	  ttl_state_base_uri(State, Base),
 	  atom_concat(Base, Name, Type).
 mk_object(Value, _State, literal(Value)).
 
-syntax_error(State, Error) -->
-	[ Before ],
-	{ arg(7, State, Stream),
+syntax_rule(State, Error) -->
+	error_tokens(7, Tokens),
+	{ ttl_state_input(State, Stream),
 	  stream_property(Stream, file_name(File)),
-	  arg(8, State, LineNo),
-	  format(string(Msg), '~w:~d (before ~w)',
+	  ttl_state_line_no(State, LineNo),
+	  atomic_list_concat(Tokens, ' ', Before),
+	  format(string(Msg), '~w:~d (before "~w ...")',
 		 [File, LineNo, Before]),
 	  throw(error(syntax_error(Error),
 		      context(_, Msg)))
 	}.
 
+%%	error_tokens(+Count, -Tokens) is det.
+%
+%	Return maximum Count tokens,  converted   back  to  turtle input
+%	syntax.
+
+error_tokens(N, [H|T]) -->
+	{ succ(N2, N) },
+	error_token(H), !,
+	error_tokens(N2, T).
+error_tokens(_, []) --> [].
+
+error_token(Name) -->
+	[ name(Name) ], !.
+error_token(Text) -->
+	[ numeric(_, Codes) ], !,
+	{ atom_codes(Text, Codes) }.
+error_token(Text) -->
+	[ literal(Literal) ], !,
+	{ literal_text(Literal, Text) }.
+error_token(Text) -->
+	[ URIToken ],
+	{ uri_text(URIToken, Text) }, !.
+error_token(Punct) -->
+	[ Punct ],
+	{ atom(Punct) }, !.
+error_token(Rest) -->
+	[ H ],
+	{ term_to_atom(H, Rest) }.
+
+literal_text(type(Type, Value), Text) :- !,
+	uri_text(Type, TypeText),
+	format(atom(Text), '"~w"^^~w', [Value, TypeText]).
+
+uri_text(relative_uri(URI), Text) :-
+	format(atom(Text), '<~w>', [URI]).
+uri_text(:(Name), Text) :-
+	format(atom(Text), ':~w', [Name]).
+
 
 		 /*******************************
 		 *	     TOKENISER		*
 		 *******************************/
 
+%%	read_turtle_tokens(+In, -List, +State) is det.
+%
+%	Read  the  next  Turtle  statement  as  a  list  of  tokens.  If
+%	on_error(warning)  is  active,  failure  prints  a  message  and
+%	continues reading the next statements.
+%
+%	The line_no property of the state is set to the start-line
+%
+%	@error syntax_error(Culprit)
+
+read_turtle_tokens(In, List, State) :-
+	ttl_state_on_error(State, error), !,
+	line_count(In, LineNo),
+	nb_set_line_no_of_ttl_state(LineNo, State),
+	(   turtle_tokens(In, List)
+	->  true
+	;   syntax_error_term(In, LineNo, illegal_token, Error),
+	    throw(Error)
+	).
+read_turtle_tokens(In, List, State) :-
+	line_count(In, LineNo),
+	nb_set_line_no_of_ttl_state(LineNo, State),
+	(   catch(turtle_tokens(In, List), Error, true)
+	->  (   var(Error)
+	    ->  true
+	    ;   print_message(error, Error),
+		skip_statement(In),
+		read_turtle_tokens(In, List, State)
+	    )
+	;   syntax_error_term(In, LineNo, illegal_token, Error),
+	    print_message(error, Error),
+	    skip_statement(In),
+	    read_turtle_tokens(In, List, State)
+	).
+
+%%	skip_statement(+In)
+%
+%	Skip to the end of the statement
+
+skip_statement(In) :-
+	get_code(In, C0),
+	skip_statement(C0, In).
+
+skip_statement(-1, _) :- !.
+skip_statement(0'., In) :-
+	get_code(In, C),
+	(   turtle_ws(C)
+	->  !
+	;   skip_statement(C, In)
+	).
+skip_statement(_, In) :-
+	get_code(In, C),
+	skip_statement(C, In).
+
 %%	turtle_tokens(+In, -List)
-%	
+%
 %	Read a statement from a turtle file, returning the contents as a
 %	list of tokens.
 
@@ -427,8 +680,7 @@ turtle_tokens(C0, In, List) :-
 	).
 
 turtle_token(-1, _, -1, end_of_file) :- !.
-turtle_token(0'., In, C, '.') :- !,
-	get_code(In, C).
+turtle_token(0'., _, end, '.') :- !.	% Turtle does not demand a space here!
 turtle_token(0'#, In, C, Token) :- !,
 	get_code(In, C1),
 	skip_line(C1, In, C2),
@@ -445,40 +697,38 @@ turtle_token(0'-, In, C, Number) :- !,
 turtle_token(0'+, In, C, Number) :- !,
 	turtle_number(0'+, In, C, Number).
 turtle_token(0'", In, C, Literal) :- !,
-	get_code(In, C1),
-	turtle_string(C1, In, C2, Codes),
-	atom_codes(Atom, Codes),
-	(   C2 == 0'@
-	->  get_code(In, C3),
-	    language(C3, In, C, LangCodes),
+	turtle_read_string(0'", In, C1, Atom),
+	(   C1 == 0'@
+	->  get_code(In, C2),
+	    language(C2, In, C, LangCodes),
 	    atom_codes(LangId, LangCodes),
 	    Literal = literal(lang(LangId, Atom))
-	;   C2 == 0'^,
+	;   C1 == 0'^,
 	    peek_code(In, 0'^)
 	->  get_code(In, 0'^),
-	    get_code(In, C3),
-	    resource_token(C3, In, C, Type),
+	    get_code(In, C2),
+	    resource_token(C2, In, C, Type),
 	    Literal = literal(type(Type, Atom))
-	;   C = C2,
+	;   C = C1,
 	    Literal = literal(Atom)
 	).
 turtle_token(0'_, In, C, nodeId(NodeID)) :-
 	peek_code(In, 0':), !,
 	get_code(In, _),
 	get_code(In, C1),
-	name(C1, In, C, NodeID).
+	turtle_read_name(C1, In, C, NodeID).
 turtle_token(0'<, In, C, URI) :- !,
 	resource_token(0'<, In, C, URI).
 turtle_token(0':, In, C, URI) :- !,
 	resource_token(0':, In, C, URI).
 turtle_token(C0, In, C, Token) :-
-	name(C0, In, C1, Name), !,
+	turtle_read_name(C0, In, C1, Name), !,
 	(   C1 == 0':,
 	    \+ sub_atom(Name, 0, _, _, '_'),
 	    peek_code(In, C2),
-	    name_start_char(C2)
+	    turtle_name_start_char(C2)
 	->  get_code(In, C2),
-	    name(C2, In, C, Name2),
+	    turtle_read_name(C2, In, C, Name2),
 	    Token = (Name:Name2)
 	;   Token = name(Name),
 	    C = C1
@@ -488,7 +738,7 @@ turtle_token(Punct, In, C, P) :-
 	get_code(In, C).
 
 %%	turtle_number(+Char0, +In, -CharNext, -Value)
-%	
+%
 %	Value is Type:CodeList
 
 turtle_number(0'-, In, CN, numeric(T, [0'-|Codes])) :- !,
@@ -501,7 +751,7 @@ turtle_number(C0, In, CN, Value) :-
 	turtle_number_nn(C0, In, CN, Value).
 
 turtle_number_nn(C, In, CN, numeric(Type, Codes)) :-
-	turtle_integer_codes(C, In, CN0, Codes, T0), 	% [0-9]+ 
+	turtle_integer_codes(C, In, CN0, Codes, T0),	% [0-9]+
 	(   CN0 == 0'.
 	->  T0 = [CN0|T1],
 	    get_code(In, C1),
@@ -542,84 +792,16 @@ e(0'E).
 sign(0'-).
 sign(0'+).				%'
 
-%%	turtle_string(+Char, +In, -NextChar, -String) is det.
-%
-%	Extract a turtle string. We have  seen   a  single  ". There are
-%	-like Python- two types  of  turtle   strings,  one  written  as
-%	"hello" and the other as """hello""".
-
-turtle_string(0'", In, Next, String) :-
-	get_code(In, C),
-	(   C == 0'"
-	->  get_code(In, C2),
-	    turtle_q_string(C2, In, Next, q3, String)
-	;   Next = C,
-	    String = []
-	).
-turtle_string(C, In, Next, String) :-
-	turtle_q_string(C, In, Next, q1, String).
-
-
-turtle_q_string(-1, In, _, _, []) :- !,
-	syntax_error(In, -1, unexpected_end_of_input).
-turtle_q_string(0'", In, C, Q, String) :- !,
-	get_code(In, C2),
-	(   Q == q1
-	->  C = C2,
-	    String = []
-	;   /* Q == q3 */
-	    C2 == 0'"
-        ->  get_code(In, C3),
-	    (	C3 == 0'"
-	    ->  get_code(In, C),
-	        String = []
-	    ;   String = [0'", 0'"|Rest],
-	        turtle_q_string(C3, In, C, q3, Rest)
-	    )
-	;   String = [0'"|Rest],
-	    turtle_q_string(C2, In, C, q3, Rest)
-	).
-turtle_q_string(0'\\, In, C, Q, [H|T]) :- !,
-	get_code(In, C1),
-	string_escape(C1, In, C2, H),
-	turtle_q_string(C2, In, C, Q, T).
-turtle_q_string(C0, In, C, Q, [C0|T]) :-
-	get_code(In, C1),
-	turtle_q_string(C1, In, C, Q, T).
-
-
-string_escape(0'n, In, C, 0'\n) :- !,
-	get_code(In, C).
-string_escape(0'", In, C, 0'") :- !,
-	get_code(In, C).
-string_escape(0'\\, In, C, 0'\\) :- !,
-	get_code(In, C).
-string_escape(0't, In, C, 0'\t) :- !,
-	get_code(In, C).
-string_escape(0'r, In, C, 0'\r) :- !,
-	get_code(In, C).
-string_escape(0'u, In, C, Code) :- !,
-	get_hhhh(In, Code),
-	get_code(In, C).
-string_escape(0'U, In, C, Code) :- !,
-	get_hhhh(In, Code0),
-	get_hhhh(In, Code1),
-	Code is Code0 << 16 + Code1,
-	get_code(In, C).
-
-get_hhhh(In, Code) :-
-	get_code(In, C1), code_type(C1, xdigit(D1)),
-	get_code(In, C2), code_type(C2, xdigit(D2)),
-	get_code(In, C3), code_type(C3, xdigit(D3)),
-	get_code(In, C4), code_type(C4, xdigit(D4)),
-	Code is D1<<12+D2<<8+D3<<4+D4.
 
 					% language: [a-z]+ ('-' [a-z0-9]+ )*
 language(C0, In, C, [C0|Codes]) :-
 	code_type(C0, lower),
 	get_code(In, C1),
 	lwr_word(C1, In, C2, Codes, Tail),
-	sub_langs(C2, In, C, Tail, []).
+	sub_langs(C2, In, C, Tail, []), !.
+language(_, In, _, _) :-
+	line_count(In, LineNo),
+	syntax_error(In, LineNo, language_specifier).
 
 lwr_word(C0, In, C, [C0|T0], T) :-
 	code_type(C0, lower), !,
@@ -648,72 +830,22 @@ lwrdigs(C, _, C, T, T).
 
 					% resource_token
 resource_token(0'<, In, C, relative_uri(URI)) :- !,
-	get_code(In, C1),
-	uri_chars(C1, In, C, Codes),
-	atom_codes(URI, Codes).
+	turtle_read_relative_uri(0'<, In, C, URI).
 resource_token(0':, In, C, Token) :- !,
 	get_code(In, C0),
-	(   name(C0, In, C, Name)
+	(   turtle_read_name(C0, In, C, Name)
 	->  Token = :(Name)
 	;   Token = :,
 	    C = C0
 	).
 resource_token(C0, In, C, Prefix:Name) :-
-	name(C0, In, C1, Prefix),
+	turtle_read_name(C0, In, C1, Prefix),
 	\+ sub_atom(Prefix, 0, _, _, '_'), !,
 	C1 == 0':,
 	get_code(In, C2),
-	name(C2, In, C, Name).
+	turtle_read_name(C2, In, C, Name).
 
 
-%%	uri_chars(+Char, +In:stream, -NextChar, -UriChars) is semidet.
-
-uri_chars(0'>, In, C, []) :- !,
-	get_code(In, C).
-uri_chars(0'\\, In, C, [H|T]) :- !,
-	get_code(In, C1),
-	string_escape(C1, In, C2, H),
-	uri_chars(C2, In, C, T).
-uri_chars(-1, _, _, _) :- !,
-	fail.
-uri_chars(C0, In, C, [C0|T]) :-
-	get_code(In, C1),
-	uri_chars(C1, In, C, T).
-
-					% name
-name(C0, In, C, Atom) :-
-	name_start_char(C0),
-	get_code(In, C1),
-	name_chars(C1, In, C, T),
-	atom_codes(Atom, [C0|T]).
-
-name_chars(C0, In, C, [C0|T]) :-
-	name_char(C0), !,
-	get_code(In, C1),
-	name_chars(C1, In, C, T).
-name_chars(C, _, C, []).
-
-name_start_char(C) :- code_type(C, csymf).
-name_start_char(C) :- between(0xC0, 0xD6, C).
-name_start_char(C) :- between(0xD8, 0xF6, C).
-name_start_char(C) :- between(0xF8, 0x2FF, C).
-name_start_char(C) :- between(0x370, 0x37D, C).
-name_start_char(C) :- between(0x37F, 0x1FFF, C).
-name_start_char(C) :- between(0x200C, 0x200D, C).
-name_start_char(C) :- between(0x2070, 0x218F, C).
-name_start_char(C) :- between(0x2C00, 0x2FEF, C).
-name_start_char(C) :- between(0x3001, 0xD7FF, C).
-name_start_char(C) :- between(0xF900, 0xFDCF, C).
-name_start_char(C) :- between(0xFDF0, 0xFFFD, C).
-name_start_char(C) :- between(0x10000, 0xEFFFF, C).
-
-name_char(C) :-	name_start_char(C).
-name_char(0'-).
-name_char(D) :-	code_type(D, digit).
-name_char(0xB7).
-name_char(C) :- between(0x0300, 0x036F, C).
-name_char(C) :- between(0x203F, 0x2040, C).
-
 punctuation(0'(, '(').
 punctuation(0'), ')').
 punctuation(0'[, '[').
@@ -728,7 +860,8 @@ skip_line(0xA, In, C) :- !,
 	get_code(In, C).
 skip_line(0xD, In, C) :- !,
 	get_code(In, C).
-skip_line(_, In, C) :- !,
+skip_line(-1, _, -1) :- !.
+skip_line(_, In, C) :-
 	get_code(In, C1),
 	skip_line(C1, In, C).
 
@@ -737,18 +870,29 @@ turtle_ws(0x9).
 turtle_ws(0xA).
 turtle_ws(0xD).
 turtle_ws(0x20).
+turtle_ws(0xFEFF).			% Unicode BOM marker and zero-width space
 
-syntax_error(Stream, -1, Which) :- !,
-	stream_property(Stream, file_name(File)),
+syntax_error(Stream, Line, Which) :-
+	syntax_error_term(Stream, Line, Which, Error),
+	throw(Error).
+
+syntax_error_term(Stream, -1, Which, Error) :- !,
+	stream_file_name(Stream, File),
 	line_count(Stream, LineNo),
 	line_position(Stream, LinePos),
 	character_count(Stream, CharIndex),
-	throw(error(syntax_error(Which),
-		    file(File, LineNo, LinePos, CharIndex))).
-syntax_error(Stream, LineNo, Which) :-
-	stream_property(Stream, file_name(File)),
-	throw(error(syntax_error(Which),
-		    file(File, LineNo, -1, -1))).
+	Error = error(syntax_error(Which),
+		      file(File, LineNo, LinePos, CharIndex)).
+syntax_error_term(Stream, LineNo, Which, Error) :-
+	stream_file_name(Stream, File),
+	Error = error(syntax_error(Which),
+		      file(File, LineNo, -1, -1)).
+
+stream_file_name(Stream, File) :-
+	stream_property(Stream, file_name(File)), !.
+stream_file_name(_, '<no file>').
+
+
 
 		 /*******************************
 		 *	    RDF-DB HOOK		*
@@ -758,8 +902,8 @@ syntax_error(Stream, LineNo, Which) :-
 	rdf_db:rdf_load_stream/3,
 	rdf_db:rdf_file_type/2.
 
-rdf_db:rdf_load_stream(turtle, Stream, Options) :-
-	option(db(Id), Options),
+rdf_db:rdf_load_stream(turtle, Stream, _Module:Options) :-
+	rdf_db:graph(Options, Id),
 	rdf_transaction(rdf_process_turtle(Stream, assert_triples, Options),
 			parse(Id)).
 
@@ -769,13 +913,5 @@ assert_triples([rdf(S,P,O)|T], Location) :-
 	assert_triples(T, Location).
 
 rdf_db:rdf_file_type(ttl, turtle).
-
-
-		 /*******************************
-		 *	   XREF SUPPORT		*
-		 *******************************/
-
-:- multifile
-	prolog:meta_goal/2.
-
-prolog:meta_goal(rdf_process_turtle(_,G,_), [G+2]).
+rdf_db:rdf_file_type(n3,  turtle).	% not really, but good enough
+rdf_db:rdf_file_type(nt,  turtle).	% not really, but good enough
