@@ -42,6 +42,7 @@
 	    current_setting/1,		% Module:Name
 	    setting_property/2,		% ?Setting, ?Property
 	    list_settings/0,
+	    list_settings/1,		% +Module
 
 	    convert_setting_text/3	% +Type, +Text, -Value
 	  ]).
@@ -49,6 +50,7 @@
 :- use_module(library(broadcast)).
 :- use_module(library(debug)).
 :- use_module(library(option)).
+:- use_module(library(arithmetic)).
 
 /** <module> Setting management
 
@@ -94,14 +96,14 @@ access, loading and saving of settings.
 	current_setting(:),
 	restore_setting(:).
 
-curr_setting(Name, Module, Type, Default, Comment) :-
-	current_setting(Name, Module, Type, Default0, Comment, _Src),
+curr_setting(Name, Module, Type, Default, Comment, Src) :-
+	current_setting(Name, Module, Type, Default0, Comment, Src),
 	(   st_default(Name, Module, Default1)
 	->  Default = Default1
 	;   Default = Default0
 	).
 
-%%	setting(Name, Type, Default, Comment) is det.
+%%	setting(:Name, +Type, +Default, +Comment) is det.
 %
 %	Define a setting. Name denotes the name of the setting, Type its
 %	type. Default is the value before  it is modified. Default refer
@@ -158,7 +160,7 @@ setting(QName, Value) :-
 	(   ground(Name)
 	->  (   st_value(Name, Module, Value0)
 	    ->  Value = Value0
-	    ;   curr_setting(Name, Module, Type, Default, _)
+	    ;   curr_setting(Name, Module, Type, Default, _, _)
 	    ->	eval_default(Default, Module, Type, Value)
 	    ;	existence_error(setting, Module:Name)
 	    )
@@ -235,7 +237,7 @@ eval_default(setting(Name), Module, Type, Value) :- !,
 	must_be(Type, Value).
 eval_default(Expr, _, Type, Value) :-
 	numeric_type(Type, Basic), !,
-	Val0 is Expr,
+	arithmetic_expression_value(Expr, Val0),
 	(   Basic == float
 	->  Val is float(Val0)
 	;   Basic = integer
@@ -339,7 +341,7 @@ numeric_type(between(L,_), Type) :-
 set_setting(QName, Value) :-
 	strip_module(QName, Module, Name),
 	must_be(atom, Name),
-	(   curr_setting(Name, Module, Type, Default0, _Comment),
+	(   curr_setting(Name, Module, Type, Default0, _Comment, _Src),
 	    eval_default(Default0, Module, Type, Default)
 	->  (   Value == Default
 	    ->	retract_setting(Module:Name)
@@ -470,7 +472,7 @@ read_setting(In, Term) :-
 %	Store setting loaded from file in the Prolog database.
 
 store_setting(setting(Module:Name, Value), _) :-
-	curr_setting(Name, Module, Type, Default0, _Commentm), !,
+	curr_setting(Name, Module, Type, Default0, _Commentm, _Src), !,
 	eval_default(Default0, Module, Type, Default),
 	(   Value == Default
 	->  true
@@ -519,13 +521,13 @@ write_setting_header(Out) :-
 	format(Out, '*/~n~n', []).
 
 save_setting(Out, Module:Name) :-
-	curr_setting(Name, Module, Type, Default, Comment),
+	curr_setting(Name, Module, Type, Default, Comment, _Src),
 	(   st_value(Name, Module, Value),
 	    \+ ( eval_default(Default, Module, Type, DefValue),
 		 debug(setting, '~w <-> ~w~n', [DefValue, Value]),
 	         DefValue =@= Value
 	       )
-	->  format(Out, '~n%	~w~n', [Comment]),
+	->  format(Out, '~n%\t~w~n', [Comment]),
 	    format(Out, 'setting(~q:~q, ~q).~n', [Module, Name, Value])
 	;   true
 	).
@@ -552,42 +554,60 @@ current_setting(Module:Name) :-
 %		* default(-Default)
 %		Default value.  If this is an expression, it is
 %		evaluated.
+%		* source(-File:-Line)
+%		Location where the setting is defined.
 
 setting_property(Setting, Property) :-
 	ground(Setting), !,
 	Setting = Module:Name,
-	curr_setting(Name, Module, Type, Default, Comment), !,
-	setting_property(Property, Module, Type, Default, Comment).
+	curr_setting(Name, Module, Type, Default, Comment, Src), !,
+	setting_property(Property, Module, Type, Default, Comment, Src).
 setting_property(Setting, Property) :-
 	Setting = Module:Name,
-	curr_setting(Name, Module, Type, Default, Comment),
-	setting_property(Property, Module, Type, Default, Comment).
+	curr_setting(Name, Module, Type, Default, Comment, Src),
+	setting_property(Property, Module, Type, Default, Comment, Src).
 
-setting_property(type(Type),       _, Type, _,        _).
-setting_property(default(Default), M, Type, Default0, _) :-
+setting_property(type(Type), _, Type, _, _, _).
+setting_property(default(Default), M, Type, Default0, _, _) :-
 	eval_default(Default0, M, Type, Default).
-setting_property(comment(Comment), _, _,    _,        Comment).
+setting_property(comment(Comment), _, _, _, Comment, _).
+setting_property(source(Src), _, _, _, _, Src).
 
-%%	list_settings
+%%	list_settings is det.
+%%	list_settings(+Module) is det.
 %
-%	List settings to =current_output=.
+%	List settings to =current_output=. The   second  form only lists
+%	settings on the matching module.
+%
+%	@tbd	Compute the required column widths
 
 list_settings :-
-	format('~`=t~72|~n'),
-	format('~w~t~20| ~w~w~t~40| ~w~n', ['Name', 'Value (*=modified)', '', 'Comment']),
-	format('~`=t~72|~n'),
-	forall(current_setting(Module:Setting),
-	       list_setting(Module:Setting)).
+	list_settings(_).
 
-list_setting(Module:Name) :-
-	curr_setting(Name, Module, Type, Default0, Comment),
+list_settings(Spec) :-
+	spec_term(Spec, Term),
+	TS1 = 25,
+	TS2 = 40,
+	format('~`=t~72|~n'),
+	format('~w~t~*| ~w~w~t~*| ~w~n',
+	       ['Name', TS1, 'Value (*=modified)', '', TS2, 'Comment']),
+	format('~`=t~72|~n'),
+	forall(current_setting(Term),
+	       list_setting(Term, TS1, TS2)).
+
+spec_term(M:S, M:S) :- !.
+spec_term(M, M:_).
+
+
+list_setting(Module:Name, TS1, TS2) :-
+	curr_setting(Name, Module, Type, Default0, Comment, _Src),
 	eval_default(Default0, Module, Type, Default),
 	setting(Module:Name, Value),
 	(   Value \== Default
 	->  Modified = (*)
 	;   Modified = ''
 	),
-        format('~w~t~20| ~q~w~t~40| ~w~n', [Module:Name, Value, Modified, Comment]).
+        format('~w~t~*| ~q~w~t~*| ~w~n', [Module:Name, TS1, Value, Modified, TS2, Comment]).
 
 
 		 /*******************************
