@@ -16,6 +16,7 @@ index((emacs_tag), 3, emacs_tags, emacs_tags).
 index((emacs_tag_file), 1, emacs_tags, emacs_tags).
 index((emacs_init_tags), 1, emacs_tags, emacs_tags).
 index((emacs_complete_tag), 2, emacs_tags, emacs_tags).
+index((getpass), 1, getpass, getpass).
 index((guitracer), 0, gui_tracer, gui_tracer).
 index((noguitracer), 0, gui_tracer, gui_tracer).
 index((prolog_break_at), 3, gui_tracer, gui_tracer).
@@ -139,7 +140,6 @@ index((xref_clean), 1, pce_prolog_xref, pce_prolog_xref).
 index((xref_current_source), 1, pce_prolog_xref, pce_prolog_xref).
 index((xref_done), 2, pce_prolog_xref, pce_prolog_xref).
 index((xref_built_in), 1, pce_prolog_xref, pce_prolog_xref).
-index((xref_expand), 2, pce_prolog_xref, pce_prolog_xref).
 index((xref_source_file), 3, pce_prolog_xref, pce_prolog_xref).
 index((xref_source_file), 4, pce_prolog_xref, pce_prolog_xref).
 index((xref_public_list), 4, pce_prolog_xref, pce_prolog_xref).
@@ -179,6 +179,7 @@ index((save_pcedraw), 1, pce_draw, pcedraw).
 index((portray_object), 1, pce_portray_object, portray_object).
 index((portray_object), 2, pce_portray_object, portray_object).
 index((qcompile_pce), 0, pce_qrecompile, qrecompile).
+index((rdf_diagram_from_file), 1, rdf_diagram, rdf_diagram).
 index((scan_arguments), 2, scan_arguments, scan_arguments).
 index((scan_arguments), 3, scan_arguments, scan_arguments).
 index((pce_fcompile), 1, sp_fcompile, sp_fcompile).
@@ -199,6 +200,7 @@ index((prolog_help), 1, pui_help, swi_help).
 index((prolog_apropos), 1, pui_help, swi_help).
 index((prolog_help_topic), 1, pui_help, swi_help).
 index((prolog_explain), 1, pui_help, swi_help).
+index((prolog_ide), 0, swi_ide, swi_ide).
 index((prolog_ide), 1, swi_ide, swi_ide).
 index((prolog_edit_preferences), 1, prolog_preferences, swi_preferences).
 index((prolog_debug_monitor), 0, pce_debug_monitor, 'swi/pce_debug_monitor').
