@@ -105,6 +105,7 @@ pce_ifhostproperty(prolog(swi),
 :- declare_emacs_mode(prolog,	library('emacs/prolog_mode')).
 :- declare_emacs_mode(chr,	library('emacs/chr_mode')).
 :- declare_emacs_mode(latex,	library('emacs/latex_mode')).
+:- declare_emacs_mode(logtalk,	library('emacs/logtalk_mode')).
 %:- declare_emacs_mode(html,	library('emacs/html_mode')).
 :- declare_emacs_mode(java,	library('emacs/java_mode')).
 :- declare_emacs_mode(c,	library('emacs/c_mode')).
@@ -134,7 +135,6 @@ pce_ifhostproperty(prolog(swi),
 	      new(hash_table)).			  % name --> mode object
 :- pce_global(@emacs,
 	      new(emacs(@emacs_buffers))).
-:- pce_global(@emacs_comment_column, new(number(40))).
 :- pce_global(@emacs_default_mode, new(var(value := script))).
 :- pce_global(@emacs_mode_list, make_emacs_mode_list).
 :- pce_global(@emacs_interpreter_mode_list, make_emacs_interpreter_mode_list).
@@ -161,9 +161,11 @@ make_emacs_mode_list(Sheet) :-
 default_emacs_mode('.*\\.pl~?$',		   prolog).
 default_emacs_mode('.*\\.plu~?$',		   prolog).
 default_emacs_mode('\\.(pl|xpce|pceemacs)rc~?',	   prolog).
+default_emacs_mode('\\.yap~?',			   prolog).
 default_emacs_mode('.*\.chr~?$',		   chr).
 default_emacs_mode('.*\\.(tex|sty)~?$',		   latex).
 default_emacs_mode('.*\\.doc~?$',		   latex).
+default_emacs_mode('.*\\.lgt~?$',		   logtalk).
 default_emacs_mode('.*\\.html~?$',		   html).
 default_emacs_mode('.*\\.php[0-9]?~?$',		   html).
 default_emacs_mode('.*\\.sgml~?$',		   sgml).
@@ -221,7 +223,7 @@ make_emacs_content_mode_list(Sheet) :-
 %	True if Mode must be used for a   file in which Regex matches in
 %	the first Limit characters of the file.
 
-emacs_content_mode('library(chr)', 	5000,	chr).
+emacs_content_mode('library(chr)',	5000,	chr).
 
 
 %	Do not make backup of a file matching this pattern
@@ -232,14 +234,15 @@ make_no_backup_list(Ch) :-
 	->  CaseSensitive = @on
 	;   CaseSensitive = @off
 	),
-	no_backup(Pattern),
-	send(Ch, append, regex(Pattern, CaseSensitive)).
+	forall(no_backup(Pattern),
+	       send(Ch, append, regex(Pattern, CaseSensitive))).
 
 %%	no_backup(?Regex)
 %
 %	True if PceEmacs does not backup files that match Regex.
 
 no_backup('/tmp/.*').
+no_backup('.*/COMMIT_EDITMSG$').
 
 :- free(@loading_emacs).
 
