@@ -56,6 +56,8 @@
 	  write_region		   = key('\\e\\C-w'),
 	  compile		   = key('\\C-x\\C-m'),
 
+	  prefix		   = key('\\eg'),
+
 					% FILE menu
 	  show_buffer_menu	   = key('\\C-x\\C-b') + button(file),
 	  switch_to_buffer	   = key('\\C-xb') +
@@ -74,7 +76,7 @@
 	  ispell		   = button(file),
 	  shell			   = button(file),
 	  (mode)		   = key('\\em') +
-	  			     button(file, @emacs_mode?modes),
+				     button(file, @emacs_mode?modes),
 	  properties		   = button(file),
 	  -			   = button(file),
 	  quit			   = key('\\C-x\\C-c') + button(file),
@@ -86,8 +88,11 @@
 	  cut			   = button(edit),
 	  paste			   = button(edit),
 	  -			   = button(edit),
-	  find			   = button(edit),
+	  goto_line		   = button(edit) +
+				     key('\\eg\\eg') + key('\\egg'),
+	  find			   = key(key_top_4) + button(edit),
 	  replace		   = button(edit),
+	  preferences		   = button(edit),
 
 					% BROWSER menu
 	  split_window		   = key('\\C-x2') + button(browse),
@@ -106,7 +111,9 @@
 	  help_on_mode		   = button(help),
 	  customise		   = button(help),
 	  -			   = button(help),
-	  show_key_bindings	   = key('\\C-hb') + button(help),
+	  show_key_bindings	   = key(key_top_1)
+					+ key('\\C-hb')
+					+ button(help),
 	  describe_key		   = key('\\C-hk') + button(help),
 	  describe_function	   = key('\\C-hf') + button(help),
 	  -			   = button(help),
@@ -130,6 +137,9 @@ class_variable(print_command,	string,
 		 windows('NOTEPAD.EXE /P %s')
 	       ],
 	       "Command to print a file").
+class_variable(tab_width,	int,
+	       8,
+	       "Distance between tab-stops").
 
 variable(print_command, name, both,
 	 "Command to print a file").
@@ -162,7 +172,7 @@ auto_colourise_buffer(M) :->
 	    get(TB, size, Size),
 	    get(M, auto_colourise_size_limit, Limit),
 	    Size < Limit,
-	    get(M, focus_function, @nil) 	% don't colour in special modes
+	    get(M, focus_function, @nil)	% don't colour in special modes
 	->  (	send(M, colourise_buffer)
 	    ->	send(TB, coloured_generation, Generation)
 	    ;	true
@@ -222,6 +232,10 @@ quit(M) :->
 	ignore(send(M?text_buffer, save_if_modified)),
 	send(M, destroy).
 
+preferences(_M) :->
+	"Edit (xpce) preferences"::
+	prolog_edit_preferences(xpce).
+
 
 		 /*******************************
 		 *		REGION		*
@@ -482,6 +496,13 @@ newline_mode(M, Mode:mode={posix,dos}) :->
 	get(TB, file, File),
 	send(File, newline_mode, Mode).
 
+tab_width(M, Width:width='2..') :->
+	"Set distance between tab-stops"::
+	get(M, text_buffer, TB),
+	get(M, editor, E),
+	send(TB, tab_width, Width),
+	send(E, tab_distance, Width).
+
 
 		 /*******************************
 		 *	       PRINT		*
@@ -582,7 +603,7 @@ replace_end(M) :->
 '_query_replace_regex'(M, Id:event_id) :->
 	"Focus function for query_replace_regex"::
 	(   Id == 0'y				% replace-and-continue
- 	->  replace_match(M),
+	->  replace_match(M),
 	    ignore(replace_find_and_mark(M))
 	;   Id == 0'.				% replace and done
 	->  replace_match(M),
