@@ -3,9 +3,10 @@
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        wielemak@science.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2008, University of Amsterdam
+    E-mail:        J.Wielemaker@cs.vu.nl
+    WWW:           http://www.swi-prolog.org/projects/xpce/
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -49,6 +50,7 @@ User extension hooks.
 	identify/2,
 	term_colours/2,
 	goal_colours/2,
+	directive_colours/2,		% Term, Colours
 	goal_classification/2.
 
 :- emacs_extend_mode(prolog,
@@ -261,36 +263,29 @@ syntax_error(M, Stream, Start, E) :-
 	),
 	fail.
 
-%	fix_operators(+Term, +Src)
+%%	fix_operators(+Term, +Src) is det.
 %
 %	Fix flags that affect the  syntax,   such  as operators and some
 %	style checking options. Src is the  canonical source as required
 %	by the cross-referencer.
 
-fix_operators((:- Directive), Src) :- !,
-	process_directive(Directive, Src).
+fix_operators((:- Directive), Src) :-
+	catch(process_directive(Directive, Src), _, true), !.
 fix_operators(_, _).
 
 process_directive(style_check(X), _) :- !,
 	style_check(X).
-process_directive(system_mode(on), _) :- !,
-	style_check(+dollar).
 process_directive(op(P,T,N), _) :- !,
-	catch(emacs_push_op(P, T, N), _, true).
+	emacs_push_op(P, T, N).
 process_directive(module(_Name, Export), _) :- !,
-	(   member(op(P,A,N), Export),
-	    catch(emacs_push_op(P,A,N), _, fail),
-	    fail
-	;   true
-	).
+	forall(member(op(P,A,N), Export),
+	       emacs_push_op(P,A,N)).
 process_directive(use_module(Spec), Src) :- !,
-	process_use_module(Spec, Src).
-process_directive(Directive, _) :-
-	asserta(user:message_hook(_,_,_), Ref),
-	ignore(xref_expand((:- Directive), _)),
-	erase(Ref).
+	catch(process_use_module(Spec, Src), _, true).
+process_directive(Directive, Src) :-
+	prolog_source:expand((:-Directive), Src, _).
 
-%	process_use_module(+Imports, +Src)
+%%	process_use_module(+Imports, +Src)
 %
 %	Get the exported operators from the referenced files.
 
@@ -450,10 +445,23 @@ functor_position(list_position(F,_T,Elms,none), F-FT, Elms) :- !,
 functor_position(Pos, Pos, []).
 
 
-%	colourise_directive(+Body, +TB, +Pos)
+%%	colourise_directive(+Body, +TB, +Pos)
 %
 %	Colourise the body of a directive.
 
+colourise_directive((A,B), TB, term_position(_,_,_,_,[PA,PB])) :- !,
+	colourise_directive(A, TB, PA),
+	colourise_directive(B, TB, PB).
+colourise_directive(Body, TB, Pos) :-
+	nonvar(Body),
+	directive_colours(Body, ClassSpec-ArgSpecs), !, % specified
+	functor_position(Pos, FPos, ArgPos),
+	(   ClassSpec == classify
+	->  goal_classification(TB, Body, [], Class)
+	;   Class = ClassSpec
+	),
+	colour_item(goal(Class, Body), TB, FPos),
+	specified_items(ArgSpecs, Body, TB, ArgPos).
 colourise_directive(Body, TB, Pos) :-
 	colourise_body(Body, TB, Pos).
 
@@ -617,13 +625,17 @@ colourise_meta_args(_, _, _, _, []) :- !.
 colourise_meta_args(N, Goal, MetaArgs, TB, [P0|PT]) :-
 	arg(N, Goal, Arg),
 	arg(N, MetaArgs, MetaSpec),
-	(   expand_meta(MetaSpec, Arg, Expanded)
-	->  colourise_goal(Expanded, [], TB, P0) % TBD: recursion
-	;   colourise_term_arg(Arg, TB, P0)
-	),
+	colourise_meta_arg(MetaSpec, Arg, TB, P0),
 	NN is N + 1,
 	colourise_meta_args(NN, Goal, MetaArgs, TB, PT).
 
+colourise_meta_arg(MetaSpec, Arg, TB, Pos) :-
+	expand_meta(MetaSpec, Arg, Expanded), !,
+	colourise_goal(Expanded, [], TB, Pos). % TBD: recursion
+colourise_meta_arg(_, Arg, TB, Pos) :-
+	colourise_term_arg(Arg, TB, Pos).
+
+
 %	meta_args(+Goal, -ArgSpec)
 %
 %	Return a copy of Goal, where each meta-argument is an integer
@@ -771,7 +783,7 @@ colourise_term_arg(Atom, TB, Pos) :-			% single quoted atom
 	colour_item(quoted_atom, TB, Pos).
 colourise_term_arg(List, TB, list_position(_, _, Elms, Tail)) :- !,
 	colourise_list_args(Elms, Tail, List, TB, classify).	% list
-colourise_term_arg(Compound, TB, Pos) :- 		% compound
+colourise_term_arg(Compound, TB, Pos) :-		% compound
 	compound(Compound), !,
 	colourise_term_args(Compound, TB, Pos).
 colourise_term_arg(_, TB, string_position(F, T)) :- !,	% string
@@ -885,6 +897,20 @@ pi_to_term(Name//Arity0, Term) :-
 	Arity is Arity0 + 2,
 	functor(Term, Name, Arity).
 
+%%	colourise_prolog_flag_name(+Name, +TB, +Pos)
+%
+%	Colourise the name of a Prolog flag
+
+colourise_prolog_flag_name(Name, TB, Pos) :-
+	atom(Name), !,
+	(   current_prolog_flag(Name, _)
+	->  colour_item(flag_name(Name), TB, Pos)
+	;   colour_item(no_flag_name(Name), TB, Pos)
+	).
+colourise_prolog_flag_name(Name, TB, Pos) :-
+	colourise_term(Name, TB, Pos).
+
+
 %%	colour_item(+Class, +TB, +Pos)
 %
 %	colourise region if a style is defined for this class.
@@ -961,7 +987,8 @@ goal_classification(_, Goal, Origin, recursion) :-
 	functor(Goal, Name, Arity),
 	functor(Origin, Name, Arity), !.
 goal_classification(TB, Goal, _, How) :-
-	xref_defined(TB, Goal, How), !.
+	xref_defined(TB, Goal, How),
+	How \= public(_), !.
 goal_classification(_TB, Goal, _, Class) :-
 	goal_classification(Goal, Class), !.
 goal_classification(_TB, _Goal, _, undefined).
@@ -1021,6 +1048,7 @@ goal_colours(thread_local(_),	     built_in-[predicates]).
 goal_colours(module_transparent(_),  built_in-[predicates]).
 goal_colours(multifile(_),	     built_in-[predicates]).
 goal_colours(volatile(_),	     built_in-[predicates]).
+goal_colours(public(_),		     built_in-[predicates]).
 goal_colours(consult(_),	     built_in-[file]).
 goal_colours(include(_),	     built_in-[file]).
 goal_colours(ensure_loaded(_),	     built_in-[file]).
@@ -1038,6 +1066,9 @@ goal_colours(retract(_),	     built_in-[db]).
 goal_colours(retractall(_),	     built_in-[db]).
 goal_colours(clause(_,_),	     built_in-[db,classify]).
 goal_colours(clause(_,_,_),	     built_in-[db,classify,classify]).
+% misc
+goal_colours(set_prolog_flag(_,_),   built_in-[prolog_flag_name,classify]).
+goal_colours(current_prolog_flag(_,_), built_in-[prolog_flag_name,classify]).
 % XPCE stuff
 goal_colours(pce_autoload(_,_),	     classify-[classify,file]).
 goal_colours(pce_image_directory(_), classify-[directory]).
@@ -1094,12 +1125,12 @@ head_colours(M:_, meta-[module(M),extern(M)]).
 
 def_style(goal(built_in,_),	style(colour := blue)).
 def_style(goal(imported(_),_),	style(colour := blue)).
-def_style(goal(autoload,_), 	style(colour := navy_blue)).
+def_style(goal(autoload,_),	style(colour := navy_blue)).
 def_style(goal(global,_),	style(colour := navy_blue)).
 def_style(goal(undefined,_),	style(colour := red)).
 def_style(goal(thread_local(_),_), style(colour := magenta,
 				      underline:= @on)).
-def_style(goal(dynamic(_),_), 	style(colour := magenta)).
+def_style(goal(dynamic(_),_),	style(colour := magenta)).
 def_style(goal(multifile(_),_),	style(colour := navy_blue)).
 def_style(goal(expanded,_),	style(colour := blue,
 				      underline := @on)).
@@ -1112,23 +1143,24 @@ def_style(goal(local(_),_),	@default).
 def_style(goal(constraint(_),_), style(colour := darkcyan)).
 
 def_style(head(exported),	style(bold := @on, colour := blue)).
+def_style(head(public(_)),	style(bold := @on, colour := '#016300')).
 def_style(head(extern(_)),	style(bold := @on, colour := blue)).
 def_style(head(dynamic),	style(bold := @on, colour := magenta)).
 def_style(head(multifile),	style(bold := @on, colour := navy_blue)).
 def_style(head(unreferenced),	style(bold := @on, colour := red)).
-def_style(head(hook),	  	style(underline  := @on, colour := blue)).
-def_style(head(meta),	  	@default).
+def_style(head(hook),		style(underline  := @on, colour := blue)).
+def_style(head(meta),		@default).
 def_style(head(constraint(_)),	style(bold := @on, colour := darkcyan)).
-def_style(head(_),	  	style(bold := @on)).
+def_style(head(_),		style(bold := @on)).
 def_style(module(_),		style(colour := dark_slate_blue)).
 def_style(comment,		style(colour := dark_green)).
 
-def_style(directive,	  	style(background := grey90)).
-def_style(method(_),	  	style(bold := @on)).
+def_style(directive,		style(background := grey90)).
+def_style(method(_),		style(bold := @on)).
 
-def_style(var,		  	style(colour := red4)).
+def_style(var,			style(colour := red4)).
 def_style(unbound,		style(bold := @on, colour := red)).
-def_style(quoted_atom,        	style(colour := navy_blue)).
+def_style(quoted_atom,		style(colour := navy_blue)).
 def_style(string,		style(colour := navy_blue)).
 def_style(nofile,		style(colour := red)).
 def_style(file(_),		style(colour := blue,
@@ -1140,14 +1172,16 @@ def_style(class(library(_),_),	style(colour := navy_blue,
 				      underline := @on)).
 def_style(class(local(_,_,_),_), style(underline := @on)).
 def_style(class(user(_),_),	style(underline := @on)).
-def_style(class(user,_), 	style(underline := @on)).
+def_style(class(user,_),	style(underline := @on)).
 def_style(class(undefined,_),	style(colour := red,
 				      underline  := @on)).
 def_style(prolog_data,		style(colour := blue,
 				      underline  := @on)).
+def_style(flag_name(_),		style(colour := blue)).
+def_style(no_flag_name(_),	style(colour := red)).
 
-def_style(keyword(_), 		style(colour := blue)).
-def_style(identifier, 		style(bold := @on)).
+def_style(keyword(_),		style(colour := blue)).
+def_style(identifier,		style(bold := @on)).
 def_style(delimiter,		style(bold := @on)).
 def_style(expanded,		style(colour := blue,
 				      underline  := @on)).
@@ -1321,6 +1355,8 @@ specified_item(body, Term, TB, Pos) :- !,
 	colourise_body(Term, TB, Pos).
 specified_item(setof, Term, TB, Pos) :- !,
 	colourise_setof(Term, TB, Pos).
+specified_item(meta(MetaSpec), Term, TB, Pos) :- !,
+	colourise_meta_arg(MetaSpec, Term, TB, Pos).
 					% DCG goal in body
 specified_item(dcg, Term, TB, Pos) :- !,
 	colourise_dcg(Term, [], TB, Pos).
@@ -1342,6 +1378,9 @@ specified_item(imports(File), Term, TB, Pos) :- !,
 					% Name/Arity, ...
 specified_item(predicates, Term, TB, Pos) :- !,
 	colourise_declarations(Term, TB, Pos).
+					% set_prolog_flag(Name, _)
+specified_item(prolog_flag_name, Term, TB, Pos) :- !,
+	colourise_prolog_flag_name(Term, TB, Pos).
 					% XPCE new argument
 specified_item(pce_new, Term, TB, Pos) :- !,
 	(   atom(Term)
@@ -1643,6 +1682,8 @@ identify_pred(autoload, F, Summary) :-	% Autoloaded predicates
 	new(Summary, string('%N: autoload from %s', F, Atom)).
 identify_pred(local(Line), F, Summary) :-	% Local predicates
 	new(Summary, string('%N: locally defined at line %d', F, Line)).
+identify_pred(public(Line), F, Summary) :-	% Public predicates
+	new(Summary, string('%N: declared public at line %d', F, Line)).
 identify_pred(foreign(Line), F, Summary) :-	% Foreign predicates
 	new(Summary, string('%N: foreign (C/C++) loaded at line %d', F, Line)).
 identify_pred(constraint(Line), F, Summary) :-	% Local constraint
@@ -1907,7 +1948,8 @@ identify_fragment(module(Module), _, Summary) :-
 identify_fragment(method(send), _, 'XPCE send method').
 identify_fragment(method(get), _, 'XPCE get method').
 identify_fragment(head(unreferenced), _, 'Unreferenced predicate (from this file)').
-identify_fragment(head(exported), _, 'Exported (Public) predicate').
+identify_fragment(head(exported), _, 'Exported predicate').
+identify_fragment(head(public), _, 'Public predicate').
 identify_fragment(head(multifile), _, 'Multifile predicate').
 identify_fragment(head(constraint), _, 'Constraint').
 identify_fragment(prolog_data, _, 'Pass Prolog term unmodified').
