@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -42,10 +43,11 @@
 :- pce_autoload(float_item,   library(pce_float_item)).
 :- pce_autoload(tick_box,     library(pce_tick_box)).
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+/** <module> XPCE-base SWI-Prolog thread monitor
+
 This library defines  the  class   prolog_thread_monitor,  a  frame that
 displays the status of threads.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+*/
 
 resource(running,   image, library('trace/icons/mini-run.xpm')).
 resource(true,	    image, image('16x16/ok.xpm')).
@@ -258,12 +260,6 @@ initialise(TB) :->
 			      message(@arg1, join),
 			      condition := not(IsRunning)),
 		    gap,
-		    menu_item(attach_console,
-			      message(@arg1, signal, attach_console),
-			      condition := IsRunning),
-		    menu_item(trace,
-			      message(@arg1, trace),
-			      condition := IsRunning),
 		    menu_item(graphical_debugger,
 			      message(@arg1, gtrace),
 			      condition := IsRunning),
@@ -274,6 +270,13 @@ initialise(TB) :->
 			      message(@arg1, signal, nodebug),
 			      condition := IsRunning),
 		    gap,
+		    menu_item(trace,
+			      message(@arg1, trace),
+			      condition := IsRunning),
+		    menu_item(attach_console,
+			      message(@arg1, signal, attach_console),
+			      condition := IsRunning),
+		    gap,
 		    menu_item(profile,
 			      message(@arg1, start_profile),
 			      condition := and(IsRunning,
@@ -320,7 +323,12 @@ update(TB) :->
 	       send(TB, update_thread, Id, Status)),
 	send(Dict, for_all,
 	     if(@arg1?seen == @off,
-		message(@arg1, free))).
+		message(@arg1, free))),
+	(   get(TB, selection, TS),
+	    TS \== @nil
+	->  send(TB, report_status, TS)
+	;   true
+	).
 
 
 recall(TB, Recall:'1..') :->
@@ -355,18 +363,25 @@ details(TB, TS:thread_status) :->
 	"Show detailed usage of thread"::
 	get(TB?frame, member, thread_window, P),
 	send(P, thread_diagram, TS),
+	send(TB, report_status, TS).
+
+report_status(TB, TS:thread_status) :->
+	"Report status of a thread"::
 	get(TS, status, Status),
 	get(TS, key, TID),
-	send(TB, report_status, TID, Status).
-
-report_status(TB, TID:any, Status:prolog) :->
-	"Report status of a thread"::
+	get(TS, lastcpu, Time),
+	(   number(Time)
+	->  RTime = Time
+	;   RTime = 0
+	),
 	(   Status = exception(Except),
 	    special_exception(Except)
 	->  message_to_string(Except, Message),
-	    send(TB, report, status, 'Thread %s ERROR: %s', TID, Message)
+	    send(TB, report, status,
+		 'Thread %s ERROR: %s (%.3f sec CPU)', TID, Message, RTime)
 	;   atomic(Status)
-	->  send(TB, report, status, 'Thread %s status: %s', TID, Status)
+	->  send(TB, report, status,
+		 'Thread %s status: %s (%.3f sec CPU)', TID, Status, RTime)
 	;   new(S, text_buffer),
 	    pce_open(S, write, Out),
 	    write_term(Out, Status, [ max_depth(5),
@@ -374,7 +389,8 @@ report_status(TB, TID:any, Status:prolog) :->
 				      portray(true)
 				    ]),
 	    close(Out),
-	    send(TB, report, status, 'Thread %s status: %s', TID, S?contents)
+	    send(TB, report, status,
+		 'Thread %s status: %s (%.3f sec CPU)', TID, S?contents, RTime)
 	).
 
 special_exception(error(_,_)).
@@ -391,9 +407,10 @@ variable(graphs, chain, get, "Graphs shown").
 initialise(TD, TS:thread_status, Graphs:chain, H:int, V:int) :->
 	"Create from thread-status report"::
 	send_super(TD, initialise),
+	default_size_limit(Limit),
 	send(TD, slot, graphs, Graphs),
 	send(TD, axis,
-	     new(X, thread_axis(y, 100, 1e8, 10,  V, point(50,200)))),
+	     new(X, thread_axis(y, 100, Limit, 10,  V, point(50,200)))),
 	send(X, scale, log),
 	send(TD, axis,
 	     plot_axis(x, 0, H, 100, H, point(50, 200))),
@@ -401,6 +418,11 @@ initialise(TD, TS:thread_status, Graphs:chain, H:int, V:int) :->
 	send(TS, recall, H),
 	send(TD, update).
 
+default_size_limit(Limit) :-
+	(   current_prolog_flag(address_bits, 32)
+	->  Limit is 100*1000*1000
+	;   Limit is 10*1000*1000*1000
+	).
 
 attach(TD, TS:thread_status) :->
 	"Switch to another thread"::
@@ -513,11 +535,14 @@ label_for_value(A, Val:'int|real', Gr:graphical) :<-
 	"Generate readable labels"::
 	(   Val < 1000
 	->  S = Val
-	;   Val < 1000000
+	;   Val < 1000*1000
 	->  K is Val / 1000,
 	    new(S, string('%dK', K))
-	;   M is Val / 1000000,
+	;   Val < 1000*1000*1000
+	->  M is Val / (1000*1000),
 	    new(S, string('%dM', M))
+	;   M is Val / (1000*1000*1000),
+	    new(S, string('%dG', M))
 	),
 	get(A, tag_font, Font),
 	new(Gr, text(S, right, Font)).
@@ -593,7 +618,7 @@ graphs(Win, Graphs:chain) :->
 :- pce_begin_class(prolog_thread_monitor, persistent_frame,
 		   "Monitor thread-activity").
 
-variable(timer,	  	  timer*,     get, "Update timer").
+variable(timer,		  timer*,     get, "Update timer").
 variable(graphs,	  chain,      get, "Which graphs are shown").
 variable(update_interval, 'int|real*', get, "Update interval").
 
@@ -609,7 +634,8 @@ initialise(TM) :->
 	send(TM, fill_tool_dialog, TD),
 	send(TM, update),
 	get(TM, update_interval, Time),
-	send(TM, update_interval, Time).
+	send(TM, update_interval, Time),
+	send(TM, selection, main).
 
 unlink(TM) :->
 	send(TM, update_interval, @nil),
@@ -663,6 +689,12 @@ update(TM) :->
 	get(TM, member, thread_browser, TB),
 	send(TB, update).
 
+selection(TM, Thread:'name|int') :->
+	"Select the given thread"::
+	get(TM, member, thread_browser, TB),
+	get(TB, member, Thread, DI),
+	send(TB, selection, DI),
+	send(TB, details, DI).
 
 recall(TM, Recall:'1..') :->
 	"#samples recalled"::
